"""
    Module initialization code
    
    Used by PloneFormGen/__init__.py

"""
import fields, form, fieldset
import actionAdapter, saveDataAdapter, formMailerAdapter
import customScriptAdapter
import thanksPage
import likertField, formLikertField


""" PloneFormGen, basic Zope Product Initialization """

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

import sys
import logging
logger = logging.getLogger("PloneFormGen")


# let's fail immediately and decisively if our dependencies
# aren't installed
try:
    from Products import PythonField, TALESField, TemplateFields
except ImportError:
    logger.error("PloneFormGen requires the ScriptableFields bundle. See PloneFormGen's README.txt.")
    raise

from Products.Archetypes.public import process_types, listTypes
from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
from AccessControl import ModuleSecurityInfo

from Products.PloneFormGen.config import PROJECTNAME, \
    ADD_CONTENT_PERMISSION, CSA_ADD_CONTENT_PERMISSION, \
    MA_ADD_CONTENT_PERMISSION, SDA_ADD_CONTENT_PERMISSION, \
    SKINS_DIR, GLOBALS, PLONE_25_PUBLISHER_MONKEYPATCH

registerDirectory(SKINS_DIR + '/PloneFormGen', GLOBALS)

def initialize(context):

    import content, validators, tools, widgets
    
    # side-effect import
    import patches

    # Add our tools
    utils.ToolInit('PloneFormGen Tool',
        tools=( tools.formGenTool.FormGenTool, ), 
        icon='Form.gif',
        ).initialize(context) 


    ##########
    # Add our content types
    # A little different from the average Archetype product
    # due to the need to individualize some add permissions.
    #
    # This approach borrowed from ATContentTypes
    #
    listOfTypes = listTypes(PROJECTNAME)

    content_types, constructors, ftis = process_types(
        listOfTypes,
        PROJECTNAME)
    allTypes = zip(content_types, constructors)
    for atype, constructor in allTypes:
        kind = "%s: %s" % (PROJECTNAME, atype.archetype_name)
        if atype.portal_type == 'FormCustomScriptAdapter':
            permission = CSA_ADD_CONTENT_PERMISSION
        elif atype.portal_type == 'FormMailerAdapter':
            permission = MA_ADD_CONTENT_PERMISSION
        elif atype.portal_type == 'FormSaveDataAdapter':
            permission = SDA_ADD_CONTENT_PERMISSION
        else:
            permission = ADD_CONTENT_PERMISSION
        utils.ContentInit(
            kind,
            content_types      = (atype,),
            permission         = permission,
            extra_constructors = (constructor,),
            fti                = ftis,
            ).initialize(context)


    ModuleSecurityInfo('Products.PloneFormGen').declarePublic('PloneFormGenMessageFactory')


# Import "PloneFormGenMessageFactory as _" to create message ids
# in the ploneformgen domain
# Zope 3.1-style messagefactory module
# BBB: Zope 2.8 / Zope X3.0
try:
    from zope.i18nmessageid import MessageFactory
except ImportError:
    from messagefactory_ import PloneFormGenMessageFactory
else:
    PloneFormGenMessageFactory = MessageFactory('ploneformgen')

# Check for Plone versions
try:
    from Products.CMFPlone.migrations import v3_0
except ImportError:
    HAS_PLONE30 = False
else:
    HAS_PLONE30 = True

if not HAS_PLONE30 and PLONE_25_PUBLISHER_MONKEYPATCH:
    from monkey import installExceptionHook
    installExceptionHook()

# alias for legacy instances of PFGCaptchaField from when it was
# a separate product
try:
    import Products.PFGCaptchaField
except ImportError:
    import Products.PloneFormGen.content.fields
    import Products.PloneFormGen.validators.CaptchaValidator
    Products.PFGCaptchaField = sys.modules['Products.PFGCaptchaField'] = sys.modules['Products.PloneFormGen']
    Products.PFGCaptchaField.field = sys.modules['Products.PFGCaptchaField.field'] = sys.modules['Products.PloneFormGen.content.fields']
    Products.PFGCaptchaField.widget = sys.modules['Products.PFGCaptchaField.widget'] = sys.modules['Products.PloneFormGen.widgets.captcha']
    Products.PFGCaptchaField.validator = sys.modules['Products.PFGCaptchaField.validator'] = sys.modules['Products.PloneFormGen.validators.CaptchaValidator']
else:
    raise "Product Conflict: The functionality of PFGCaptchaField is now included within PloneFormGen.  You must remove the PFGCaptchaField product from your filesystem.  Once you do that, Zope will start and existing captcha fields will continue to work."



from Products.validation import validation, interfaces

class LinkSpamValidator:
    """ Validates whether or not a string has anything that seems link-like. For
    these purposes, we're considering the following fragments to be linky:
        "<a "
        "www"
        "http"
        ".com"
        (See Products.PloneFormGen.config's stringValidators for an unfortunate
        repeat of this logic.)
    """

    __implements__ = (interfaces.ivalidator,)

    name = 'LinkSpamValidator'

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        # validation is optional and configured on the field
        obj = kwargs.get('field')
        if not obj:
            return 1
        if hasattr(obj,'validate_no_link_spam') and obj.validate_no_link_spam:
            bad_signs = ("<a ",
                         "www.",
                         "http:",
                         ".com",
                         )
            value = value.lower()
            for s in bad_signs:
                if s in value:
                    return ("Validation failed(%(name)s): links are not allowed." %
                            { 'name' : self.name, })
        return 1

validation.register(LinkSpamValidator('isNotLinkSpam'))



#
# Integration tests. See other test modules for specific components.
#

import os, sys, email

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PloneFormGen import HAS_PLONE30
from Products.PloneFormGen.tests import pfgtc

from Products.CMFCore.utils import getToolByName

import zExceptions

if HAS_PLONE30:
    from plone.protect.authenticator import AuthenticatorView


# too lazy to see if this is already in the library somewhere
def stripWhiteSpace(multiLineString):
    return '\n'.join([s.strip() for s in multiLineString.split('\n')])

class FakeRequest(dict):

    def __init__(self, **kwargs):
        self.form = kwargs
        self['form'] = kwargs

class TestFunctions(pfgtc.PloneFormGenTestCase):
    """ tests that mostly concern functionality beyond the unit """

    def dummy_send( self, mfrom, mto, messageText ):
        self.mfrom = mfrom
        self.mto = mto
        self.messageText = messageText


    def afterSetUp(self):
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        self.folder.invokeFactory('FormFolder', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.mailhost = self.folder.MailHost
        self.mailhost._send = self.dummy_send
        self.ff1.mailer.setRecipient_email('mdummy@address.com')


    def testFgFieldsDisplayList(self):
        """ test Form Folder's fgFieldsDisplayList """

        # in v 1.0.2, this caused "'unicode' object has no attribute 'decode'" w/ Plone 2.5.1 and Zope 2.8.7
        res = self.ff1.fgFieldsDisplayList()

        self.assertEqual(len(res), 3)
        self.assertEqual( res.keys()[0], 'replyto' )
        self.failUnless( isinstance(res.values()[0], unicode) )


    def testFgFieldsDisplayListFieldset(self):
        """ Make sure fgFieldsDisplayList works for fields in fieldsets.
            Tracker #123.
        """
        
        self.ff1.invokeFactory('FieldsetFolder', 'fsf1', title='a fieldset')
        fsf1 = self.ff1.fsf1
        fsf1.invokeFactory('FormStringField', 'fsf', title='a string field')
        
        res = self.ff1.fgFieldsDisplayList(objTypes=['FormStringField'])
        self.assertEqual(len(res), 3)

        res = self.ff1.mailer.fieldsDisplayList()
        self.assertEqual(len(res), 4)


    def testFgFieldsDisplayOnly(self):
        """ test Form Folder's fgFields displayOnly option """

        ff = self.ff1
        len1 = len(ff.fgFields())

        ff.invokeFactory('FormLabelField', 'lf')
        ff.invokeFactory('FormRichLabelField', 'rlf')

        # when displayOnly==True, fgFields should not return label fields
        self.assertEqual(len(ff.fgFields(displayOnly=True)), len1)
        # when displayOnly is omitted, all fields should be returned
        self.assertEqual(len(ff.fgFields()), len1+2)


    def test_Validate(self):
        """ Test required field validation """

        request = FakeRequest(topic = 'test subject')

        errors = self.ff1.fgvalidate(REQUEST=request)
        self.failUnless( errors['replyto'] )
        self.failUnless( errors['comments'] )
        self.failUnless( errors.get('topic') is None )

        request = FakeRequest(topic = 'test subject', replyto='testtest.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.failUnless( errors['replyto'] )

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )

        # since that should have validated, it should have been mailed
        self.failUnless( self.messageText.find('Reply-To: test@test.org') > 0 )


    def test_selfValidate(self):
        """ Test field self validation """

        request = FakeRequest(topic = 'test subject ')
        errors = self.ff1.topic.fgvalidate(request)
        self.assertEqual( errors, {} )


    def test_i18nTitleValidate(self):
        """ Test field self validation with required non-ASCII field title """

        self.ff1.topic.setTitle('Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es')
        request = FakeRequest()
        errors = self.ff1.topic.fgvalidate(request)
        self.failUnless( errors.has_key('topic') )


    def test_CustomValidation(self):
        """ test to make sure the custom TALES validation works
        """

        # test in field context (field validating itself)

        # make sure it fails when test is invalid
        request = FakeRequest(topic = 'test subject ')
        self.ff1.topic.setFgTValidator( 'python: 1/0' )
        self.assertRaises(ZeroDivisionError, self.ff1.topic.fgvalidate, request)

        # now for a more realistic custom validator
        self.ff1.topic.setFgTValidator( 'python: test(value.find("test") >= 0, 0, "test is missing")' )

        request = FakeRequest(topic = 'test subject ')
        errors = self.ff1.topic.fgvalidate(request)
        self.assertEqual( errors, {} )

        request = FakeRequest(topic = 'no subject ')
        errors = self.ff1.topic.fgvalidate(request)
        self.failUnless( errors['topic'] == 'test is missing' )

        # also check in form context (form validating field)

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )

        request = FakeRequest(topic = 'no subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.failUnless( errors['topic'] == 'test is missing' )



    def testHtmlValue(self):
        """ Test field htmlValue method """

        request = FakeRequest(topic = 'test subject')
        self.assertEqual( self.ff1['topic'].htmlValue(request), 'test subject')

        # test html escaping
        request = FakeRequest(topic = 'test < & > subject')
        self.assertEqual( self.ff1['topic'].htmlValue(request), 'test &lt; &amp; &gt; subject')

        # test list cleanup
        request = FakeRequest(topic = ['one',])
        self.assertEqual( self.ff1['topic'].htmlValue(request), "'one'")

        # test non-list non-cleanup
        request = FakeRequest(topic = "['one',]")
        self.assertEqual( self.ff1['topic'].htmlValue(request), "['one',]")

        # test eol encoding
        request = FakeRequest(comments = "one\ntwo")
        self.assertEqual( self.ff1['comments'].htmlValue(request), '<div>one<br />two</div>')


    def testHtmlValueSelectionField(self):
        """ Test field htmlValue method of selection field """

        self.ff1.invokeFactory('FormSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('1|one', '2|two', '3|three',)

        # first test inside the vocabulary
        request = FakeRequest(fsf = '2')
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual( val, 'two')

        # now, outside the vocabulary
        request = FakeRequest(fsf = '7')
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual( val, '7')

        # now, outside the vocabulary;
        # make sure it's html escaped
        request = FakeRequest(fsf = '&')
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual( val, '&amp;')


    def testHtmlValueMultiSelectionField(self):
        """ Test field htmlValue method of multi-selection field """

        self.ff1.invokeFactory('FormMultiSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('1|one', '2|two', '3|three',)

        # first test inside the vocabulary
        request = FakeRequest(fsf = ['2', '3', ''])
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual( val, 'two, three')

        # now, outside the vocabulary
        request = FakeRequest(fsf = ['7', ''])
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual( val, '7')

        # now, outside the vocabulary;
        # make sure it's html-escaped
        request = FakeRequest(fsf = ['&', ''])
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual( val, '&amp;')

        # now, mixed
        request = FakeRequest(fsf = ['1', '7', ''])
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual( val, 'one, 7')


    def testHtmlValueDateField(self):
        """ Test field htmlValue method of date field """

        self.ff1.invokeFactory('FormDateField', 'fdf')

        # good date
        request = FakeRequest(fdf = '2007/01/01 00:00')
        val = self.ff1['fdf'].htmlValue(request)

        # bad date
        request = FakeRequest(fdf = '2007/01/00 00:00')
        val = self.ff1['fdf'].htmlValue(request)


    def testMissingAdapter(self):
        """ test response to missing adapter -- should not fail """

        # Note: this test logs a warning message; it does not indicate test failure

        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')

        self.ff1.setActionAdapter( ('bogus',) )
        self.assertEqual(self.ff1.actionAdapter, ('bogus',))

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )


    def testSimpleCall(self):
        """ test calling form """

        # this proves we can call a form from the test
        # harness.

        self.ff1()


    def testCallWithGoodOverride(self):
        """ test calling form with good code in override """

        # This test isn't as trivial as it looks. It makes sure the
        # call to the override doesn't clobber the expression context
        # and foul template evaluation.

        self.ff1.setOnDisplayOverride("python: 1")
        self.ff1()


    def testCallWithAfterValidationOverride(self):
        """ test calling form with good code in AfterValidationOverride """

        self.ff1.setAfterValidationOverride("python: 1")
        self.ff1()


    def testCallWithBadOverride(self):
        """ test calling form with bad code in override """

        self.ff1.setOnDisplayOverride("python: 1/0")
        self.assertRaises(ZeroDivisionError, self.ff1)


    def testCatalogCleanup(self):
        """ Test to make sure portalfactory isn't leaving
            ghost entries in portal_catalog """

        # look up all the FormStringFields
        sfb = self.portal.portal_catalog(portal_type='FormStringField')
        # There should be two
        self.assertEqual(len(sfb), 2)



    def testEmptySelectionField(self):
        """ test for issue  #80: No radio button selected: AttributeError
            'NoneType' object has no attribute 'encode' """

        self.ff1.invokeFactory('FormSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('one', 'two', 'three', 'four', 'five')
        self.ff1.fsf.setFgFormat('radio')

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        self.ff1.fsf.htmlValue(request)



    def testEmptyMultiSelectionField(self):
        """ check MSF for issue  #80: No radio button selected: AttributeError
            'NoneType' object has no attribute 'encode' """

        self.ff1.invokeFactory('FormMultiSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('one', 'two', 'three', 'four', 'five')
        self.ff1.fsf.setFgFormat('checkbox')

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        self.ff1.fsf.htmlValue(request)



    def testTrailSpacesValidation(self):
        """ check MSF for issue  #79:
            Trailing space in recipient address of mailer causes validation failure"""

        request = FakeRequest(topic = 'test subject', replyto='test@test.org ', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )



    def testTrailSpacesSave(self):
        """ We really don't want to save or act on trailing spaces in inputs """

        request = FakeRequest(topic = 'test subject', replyto='test@test.org ', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( request.form['replyto'], 'test@test.org' )



    def testWhiteSpaceInRequired(self):
        """ white-space only shouldn't validate in required fields  """

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='\n')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( len(errors), 1 )

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='xx\n')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( len(errors), 0 )

        request = FakeRequest(topic = '   ', replyto='test@test.org', comments='xx\n')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( len(errors), 1 )

        request = FakeRequest(topic = 'x   ', replyto='test@test.org', comments='xx\n')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( len(errors), 0 )


    def testListMarshallFix(self):
        """ test fgvalidate fix for odd Zope list marshalling """

        self.ff1.invokeFactory('FormMultiSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('one', 'two', 'three', 'four', 'five')
        self.ff1.fsf.setFgFormat('checkbox')
        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test', fsf=['one','two',''])
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )
        # print request.form['fsf']
        self.assertEqual( len(request.form['fsf']), 2 )


    def testFieldEnables(self):
        """ Test TAL field enables/disables  """

        # crude test: see if fields are enabled or not by testing
        # whether or not they get validated.

        # no enabling condition: should not validate
        request = FakeRequest(topic = 'test subject', replyto='test@test.org')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( len(errors), 1 )

        # explicity enabling: should not validate
        self.ff1.comments.setFgTEnabled('python: True')
        request = FakeRequest(topic = 'test subject', replyto='test@test.org')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( len(errors), 1 )

        # explicity disabling: should validate
        self.ff1.comments.setFgTEnabled('python: False')
        request = FakeRequest(topic = 'test subject', replyto='test@test.org')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( len(errors), 0 )

        # now, let's count fields returned by fgFields

        # no enabling condition: all fields should be in list
        self.ff1.comments.setFgTEnabled('')
        request = FakeRequest()
        fields = self.ff1.fgFields(request=request)
        self.assertEqual( len(fields), 3 )

        # explicitly enabled: all fields should be in list
        self.ff1.comments.setFgTEnabled('python: True')
        request = FakeRequest()
        fields = self.ff1.fgFields(request=request)
        self.assertEqual( len(fields), 3 )

        # explicitly disabled: one less fields should be in list
        self.ff1.comments.setFgTEnabled('python: False')
        request = FakeRequest()
        fields = self.ff1.fgFields(request=request)
        self.assertEqual( len(fields), 2 )


    def testTranslationBasics(self):
        """Sanity check i18n setup against some known translations"""

        from Products.PloneFormGen import PloneFormGenMessageFactory as _
        from zope.i18n import translate

        # test with:
        # msgid "clear-save-input"

        msg = _(u"clear-save-input", u"Clear Saved Input")

        xlation = translate(msg, target_language='en')
        self.assertEqual( xlation, u"Clear Saved Input" )

        xlation = translate(msg, target_language='fr')
        self.assertEqual( xlation, 'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8') )

        xlation = translate(msg, target_language='de')
        self.assertEqual( xlation, 'Die gespeicherten Eingaben l\xc3\xb6schen'.decode('utf8') )


    def testDateValidation(self):
        """ Dates should be validated """

        
        self.ff1.invokeFactory('FormDateField', 'fdf')

        # set non-date fields in request
        request = FakeRequest(topic = 'test subject', replyto='test@test.org ', comments='test comments')
        
        # try with no date at all. should validate, since fdf isn't required
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( request.form['replyto'], 'test@test.org' )
        self.assertEqual( errors, {} )

        # try with good date. should validate.
        request = FakeRequest(topic = 'test subject', replyto='test@test.org ', comments='test comments',
         fdf='2007/02/20')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( request.form['replyto'], 'test@test.org' )
        self.assertEqual( errors, {} )

        # try with bad date. should not validate.
        request = FakeRequest(topic = 'test subject', replyto='test@test.org ', comments='test comments',
         fdf='2007/02/31', fdf_year='2007', fdf_month='02', fdf_day='31')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( request.form['replyto'], 'test@test.org' )
        self.assertEqual( len(errors), 1 )

        # try required and bad date. should not validate.
        self.ff1.fdf.setRequired(True)
        request = FakeRequest(topic = 'test subject', replyto='test@test.org ', comments='test comments',
         fdf='2007/02/31', fdf_year='2007', fdf_month='02', fdf_day='31')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( request.form['replyto'], 'test@test.org' )
        self.assertEqual( len(errors), 1 )

        # try required and no date. should not validate.
        self.ff1.fdf.setRequired(True)
        request = FakeRequest(topic = 'test subject', replyto='test@test.org ', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( request.form['replyto'], 'test@test.org' )
        self.assertEqual( len(errors), 1 )

        # try required and good date. should validate.
        self.ff1.fdf.setRequired(True)
        request = FakeRequest(topic = 'test subject', replyto='test@test.org ', comments='test comments',
         fdf='2007/02/21', fdf_year='2007', fdf_month='02', fdf_day='21')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( request.form['replyto'], 'test@test.org' )
        self.assertEqual( errors, {} )

    def testEmptyHiddenLinesField(self):
        """ test empty, hidden lines field, issue 151 """

        self.ff1.invokeFactory('FormLinesField', 'flf')
        self.ff1.flf.setHidden(True)
        # smoke test
        self.ff1()

    def testActionAdapterReturns(self):
        """ test to make sure that the return status of action adapters is handled right """

        # Script code to imitate a Custom Script adapter
        # script returning something other than a dict
        non_error_script="""
        ## Python Script
        ##bind container=container
        ##bind context=context
        ##bind subpath=traverse_subpath
        ##parameters=fields, ploneformgen, request
        ##title=Succesfully working script returning error
        ##

        return False
        """

        # Script code to imitate a Custom Script adapter
        # script returning a dict and using context.FORM_ERROR_MARKER
        error_script="""
        ## Python Script
        ##bind container=container
        ##bind context=context
        ##bind subpath=traverse_subpath
        ##parameters=fields, ploneformgen, request
        ##title=Succesfully working script returning error
        ##

        return {context.FORM_ERROR_MARKER:'an error message'}
        """

        # we'll need privileges to create a script adapter
        self.loginAsPortalOwner()

        # create three adapters
        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver1')
        saver1 = self.ff1.saver1
        self.ff1.invokeFactory('FormCustomScriptAdapter', 'cscript')
        cscript = self.ff1.cscript
        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver2')
        saver2 = self.ff1.saver2
        self.ff1.setActionAdapter( ('saver1', 'cscript', 'saver2') )
        self.assertEqual(self.ff1.actionAdapter, ('saver1', 'cscript', 'saver2'))

        # fake request to fake post
        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')

        # Run a script that returns a non-error status;
        # Something should be saved to both savers,
        # and errors should be an empty dict.
        cscript.setScriptBody(stripWhiteSpace(non_error_script)) 
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {})
        self.assertEqual(saver1.itemsSaved(), 1)
        self.assertEqual(saver2.itemsSaved(), 1)

        # Run a script that returns an error status;
        # we should see an error status from the validator,
        # and action adapter execution should short-circuit
        # with only the first saver's onSuccess being executed.
        #
        # This will also demonstrate that the dictionary returned
        # by the action adapter is returned by fgvalidate
        # and that FORM_ERROR_MARKER is available as an
        # attribute of the context.
        cscript.setScriptBody(stripWhiteSpace(error_script)) 
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {cscript.FORM_ERROR_MARKER : 'an error message'})
        self.assertEqual(saver1.itemsSaved(), 2)
        self.assertEqual(saver2.itemsSaved(), 1)

    if HAS_PLONE30:
        def testCSRF(self):
            """ test csrf protection """

            # for this test, we need a bit more serious request simulation
            from ZPublisher.HTTPRequest import HTTPRequest
            from ZPublisher.HTTPResponse import HTTPResponse
            environ = {}
            environ.setdefault('SERVER_NAME', 'foo')
            environ.setdefault('SERVER_PORT', '80')
            environ.setdefault('REQUEST_METHOD',  'POST')        
            request = HTTPRequest(sys.stdin, 
                        environ,
                        HTTPResponse(stdout=sys.stdout))

            request.form = \
                 {'topic':'test subject',
                  'replyto':'test@test.org',
                  'comments':'test comments'}

            self.assertRaises(zExceptions.Forbidden, self.ff1.fgvalidate, request)
        
            # with authenticator... no error
            tag = AuthenticatorView('context', 'request').authenticator()
            token = tag.split('"')[5]
            request.form['_authenticator'] = token
            errors = self.ff1.fgvalidate(REQUEST=request)
            self.assertEqual( errors, {} )

            # sneaky GET request
            environ['REQUEST_METHOD'] = 'GET'
            request = HTTPRequest(sys.stdin, 
                        environ,
                        HTTPResponse(stdout=sys.stdout))
            self.assertRaises(zExceptions.Forbidden, self.ff1.fgvalidate, request)

            # bad authenticator
            request.form['_authenticator'] = 'inauthentic'
            request = HTTPRequest(sys.stdin, 
                        environ,
                        HTTPResponse(stdout=sys.stdout))
            self.assertRaises(zExceptions.Forbidden, self.ff1.fgvalidate, request)


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFunctions))
    return suite



#!/usr/bin/env python
# encoding: utf-8
"""
dollarReplace.py

Created by Steve McMahon on 2008-12-11.
Copyright (c) 2008, Steve McMahon. Distribution allowed under GPL2.
"""

import re

# regular expression for dollar-sign variable replacement.
# we want to find ${identifier} patterns
dollarRE = re.compile(r"\$\{(.+?)\}")

class DollarVarReplacer(object):
    """ 
    Initialize with a dictionary, then self.sub returns a string
    with all ${key} substrings replaced with values looked
    up from the dictionary.

    >>> from Products.PloneFormGen import dollarReplace

    >>> adict = {'one':'two', '_two':'three', '.two':'four'}
    >>> dvr = dollarReplace.DollarVarReplacer( adict )

    >>> dvr.sub('one one')
    'one one'

    >>> dvr.sub('one ${one}')
    'one two'

    >>> dvr.sub('one ${two}')
    'one ???'

    Skip any key beginning with _ or .
    >>> dvr.sub('one ${_two}')
    'one ???'

    >>> dvr.sub('one ${.two}')
    'one ???'

    """

    def __init__(self, adict):
        self.adict = adict

    def sub(self, s):
        return dollarRE.sub(self.repl, s)

    def repl(self, mo):
        key = mo.group(1)
        if key and key[0] not in ['_','.']:
            try:
                return self.adict[mo.group(1)]
            except KeyError:
                pass
        return '???'            

if __name__ == '__main__':
    import doctest
    doctest.testfile(__file__, verbose=True)


from zope.interface import Interface, Attribute

class IPloneFormGenThanksPage(Interface):
    """thanksPage interface
    """
    
    meta_type = Attribute("archetypes meta type")
    """
    Must match GS type declaration.
    """

    



from field import IPloneFormGenField
from form import IPloneFormGenForm
from fieldset import IPloneFormGenFieldset
from actionAdapter import IPloneFormGenActionAdapter
from thanksPage import IPloneFormGenThanksPage
from exportimport import IFormFolderExportView, IImportSchema, \
    IFormFolderImportView


from Products.Five import BrowserView

HAS_RECAPTCHA = True
try:
    from collective.recaptcha.settings import RecaptchaSettingsForm
except ImportError:
    HAS_RECAPTCHA = False

if HAS_RECAPTCHA:
    
    class CaptchaControlPanel(RecaptchaSettingsForm):
        description = u'The PloneFormGen captcha field is based on the Recaptcha service. To use it ' + \
                      u'you must obtain an account at recaptcha.net, generate keys for this site, and ' + \
                      u'enter them here. '
        
        has_recaptcha = True

else:
    
    class CaptchaControlPanel(BrowserView):
        has_recaptcha = False



"""config -- shared values"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from Products.CMFCore.permissions import setDefaultRoles
from Products.Archetypes.public import DisplayList
from Products.validation.validators.BaseValidators import protocols, EMAIL_RE


############################################
### Things you might customize for your site

DEFAULT_MAILTEMPLATE_BODY = \
"""<html xmlns="http://www.w3.org/1999/xhtml">

  <head><title></title></head>

  <body>
    <p tal:content="here/getBody_pre | nothing" />
    <dl>
        <tal:block repeat="field options/wrappedFields">
            <dt tal:content="field/fgField/widget/label" />
            <dd tal:content="structure python:field.htmlValue(request)" />
        </tal:block>
    </dl>
    <p tal:content="here/getBody_post | nothing" />
    <pre tal:content="here/getBody_footer | nothing" />
  </body>
</html>
"""

MIME_LIST = DisplayList(
    (('html', 'HTML'),
     ('plain', 'Text'),
     ))

XINFO_DEFAULT = ['HTTP_X_FORWARDED_FOR', 'REMOTE_ADDR', 'PATH_INFO']

## Customizable String Validators
#
#  These are used by the String Form Field.
#
#  All are built from a regular expression and an
#  ignore string.
#
#  Note that in Plone 2.5+, i18n messageids will
#  be constructed from the i18nid/title and errmsg/errid
#  pairs. So, these are fully translatable.

stringValidators = (
    {'id':'isEmail',
        'i18nid':'vocabulary_isemailaddress_text',
        'title':u'Is an E-Mail Address',
        'errmsg':u'This is not a valid email address.',
        'errid':'pfg_isEmail',
        'regex':'^'+EMAIL_RE,
        'ignore':'',
        },
    {'id':'isCommaSeparatedEmails',
        'i18nid':'vocabulary_isemailaddresslist_text',
        'title':u'Is one or more E-Mail Addresses separated by commas',
        'errmsg':u'This is not a valid list of email addresses (separated by commas).',
        'errid':'pfg_isEmailAddressList',
        'regex':r'^'+EMAIL_RE[:-1]+'(,\s*'+EMAIL_RE[:-1]+')*$',
        'ignore':'',
        },
    {'id':'isPrintable',
        'i18nid':'vocabulary_onlyprintable_text',
        'title':u'Contains only printable characters',
        'errmsg':u'This value contains unprintable characters.',
        'errid':'pfg_isPrintable',
        'regex':r'[a-zA-Z0-9\s]+$',
        'ignore':'',
    },
    {'id':'isURL',
        'i18nid':'vocabulary_isurl_text',
        'title':u'Is a well-formed URL',
        'errmsg':u'This is not a valid url.',
        'errid':'pfg_isURL',
        'regex':r'(%s)s?://[^\s\r\n]+' % '|'.join(protocols),
        'ignore':'',
    },
    {'id':'isUSPhoneNumber',
        'i18nid':'vocabulary_isusphone_text',
        'title':u'Is a valid US phone number',
        'errmsg':u'This is not a valid us phone number.',
        'errid':'pfg_isUSPhoneNumber',
        'regex':r'^\d{10}$',
        'ignore':'[\(\)\-\s]',
    },
    {'id':'isInternationalPhoneNumber',
        'i18nid':'vocabulary_isintphone_text',
        'title':u'Is a valid international phone number',
        'errmsg':u'This is not a valid international phone number.',
        'errid':'pfg_isInternationalPhoneNumber',
        'regex':r'^\d+$',
        'ignore':'[\(\)\-\s\+]',
    },
    {'id':'isZipCode',
        'i18nid':'vocabulary_iszipcode_text',
        'title':u'Is a valid zip code',
        'errmsg':u'This is not a valid zip code.',
        'errid':'pfg_isZipCode',
        'regex':r'^(\d{5}|\d{9})$',
        'ignore':'[\-]',
    },
    {'id': 'isNotLinkSpam',
        'i18nid':'vocabulary_isnotlinkspam_text',
        'title':u'Does not contain link spam',
        'errmsg':u'This text appears to contain links.',
        'errid':'pfg_isnotlinkspam',
        'regex':r'(?!(<a |http|www|\.com)).*',
        'ignore':'',
    }
)


######
# Extra content types allowed inside form folders

EXTRA_ALLOWED = ['Document', 'Image']

######
# LinguaPlone-related settings

# Set this True to cause SaveData action adapters in translated forms to save
# to the matching SaveData adapter in the canonical version (if there is one).
LP_SAVE_TO_CANONICAL = True


### End of likely customizations
### Change anything below and things are likely to break
########################################################


## The Project Name
PROJECTNAME = "PloneFormGen"

## The skins dir
SKINS_DIR = 'skins'

# property sheet (in portal_properties)
PROPERTY_SHEET_NAME = 'ploneformgen_properties'

## Globals variable
GLOBALS = globals()

fieldTypes = (
    'FormSelectionField',
    'FormMultiSelectionField',
    'FormLabelField',
    'FormDateField',
    'FormLinesField',
    'FormIntegerField',
    'FormBooleanField',
    'FormPasswordField',
    'FormFixedPointField',
    'FormStringField',
    'FormTextField',
    'FormRichTextField',
    'FormRichLabelField',
    'FormFileField',
    'FormLikertField',
    'FormCaptchaField',
)

adapterTypes = (
    'FormSaveDataAdapter',
    'FormMailerAdapter',
    'FormCustomScriptAdapter',    
)

thanksTypes = (
    'FormThanksPage',
)

fieldsetTypes = (
    'FieldsetFolder',
)

## Permission for content creation for most types
ADD_CONTENT_PERMISSION = 'PloneFormGen: Add Content'
setDefaultRoles(ADD_CONTENT_PERMISSION, ('Manager', 'Owner', 'Contributor',))

## Exceptions
MA_ADD_CONTENT_PERMISSION = 'PloneFormGen: Add Mailers'
setDefaultRoles(MA_ADD_CONTENT_PERMISSION, ('Manager','Owner',))
SDA_ADD_CONTENT_PERMISSION = 'PloneFormGen: Add Data Savers'
setDefaultRoles(SDA_ADD_CONTENT_PERMISSION, ('Manager','Owner',))
CSA_ADD_CONTENT_PERMISSION = 'PloneFormGen: Add Custom Scripts'
setDefaultRoles(CSA_ADD_CONTENT_PERMISSION, ('Manager',))

## Permission to use TALESField, ZPTField
EDIT_TALES_PERMISSION = 'PloneFormGen: Edit TALES Fields'
setDefaultRoles(EDIT_TALES_PERMISSION, ('Manager',))

## Permission to use TALESField, ZPTField
EDIT_PYTHON_PERMISSION = 'PloneFormGen: Edit Python Fields'
setDefaultRoles(EDIT_PYTHON_PERMISSION, ('Manager',))

## Permission to use advanced fields
EDIT_ADVANCED_PERMISSION = 'PloneFormGen: Edit Advanced Fields'
setDefaultRoles(EDIT_ADVANCED_PERMISSION, ('Manager',))

## Permission to use mail adapter addressing fields
EDIT_ADDRESSING_PERMISSION = 'PloneFormGen: Edit Mail Addresses'
setDefaultRoles(EDIT_ADDRESSING_PERMISSION, ('Manager','Owner'))

## Permission to use encryption fields
USE_ENCRYPTION_PERMISSION = 'PloneFormGen: Edit Encryption Specs'
setDefaultRoles(USE_ENCRYPTION_PERMISSION, ('Manager',))

## Permission to download saved data
DOWNLOAD_SAVED_PERMISSION = 'PloneFormGen: Download Saved Input'
setDefaultRoles(DOWNLOAD_SAVED_PERMISSION, ('Manager', 'Owner',))

## Our list of permissions
pfgPermitList = [
    ADD_CONTENT_PERMISSION,
    MA_ADD_CONTENT_PERMISSION,
    SDA_ADD_CONTENT_PERMISSION,
    CSA_ADD_CONTENT_PERMISSION,
    EDIT_TALES_PERMISSION,
    EDIT_PYTHON_PERMISSION,
    EDIT_ADVANCED_PERMISSION,
    EDIT_ADDRESSING_PERMISSION,
    USE_ENCRYPTION_PERMISSION,
    DOWNLOAD_SAVED_PERMISSION,
]

## Some object ids allowed by Plone can cause lots of trouble
#  if they're used for PFG field elements.
#  The disallow list:
BAD_IDS = ('zip', 'location', 'language')

# used to mark errors that belong to the
# whole form, not just a field.
FORM_ERROR_MARKER = '_pfg_form_error'

# apply the publisher exception hook wrapper to support embedded
# forms in Plone 2.5?
PLONE_25_PUBLISHER_MONKEYPATCH = False



""" 
    
    Execute custom Python script/external method for data
    
    Copyright 2006 Red Innovation http://www.redinnovation.com
    
"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'plaintext'

# Python imorts
from StringIO import StringIO
import logging

# Zope imports
from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent
from Products.PythonScripts.PythonScript import PythonScript


# Plone imports
from Products.Archetypes.public import *
from Products.Archetypes.utils import contentDispositionHeader
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import registerATCT
from Products.CMFCore.permissions import View, ModifyPortalContent

# Plone 3rd party imports
from Products.PythonField import PythonField as PythonField

# Local imports
from Products.PloneFormGen import HAS_PLONE30
from Products.PloneFormGen import config
from Products.PloneFormGen.config import *
from Products.PloneFormGen.content.actionAdapter import \
    FormActionAdapter, FormAdapterSchema

logger = logging.getLogger("PloneFormGen")    


# Value of new custom script body field     
default_script = """
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, ploneformgen, request
##title=
##

# Available parameters:
#  fields  = HTTP request form fields as key value pairs
#  request = The current HTTP request. 
#            Access fields by request.form["myfieldname"]
#  ploneformgen = PloneFormGen object
# 
# Return value is not processed -- unless you
# return a dictionary with contents. That's regarded
# as an error and will stop processing of actions
# and return the user to the form. Error dictionaries
# should be of the form {'field_id':'Error message'}


assert False, "Please complete your script"

"""

class FormCustomScriptAdapter(FormActionAdapter):
    """ Executes a Python script for form data
    """

    # Python script receives parameters 
    #     resultData  - cleaned up input
    #     form        - the PFG object
    #     REQUEST
    
    # External Python script (as in Plone skins)
    SCRIPT_TYPE_SKINS_SCRIPT="skins_script"
    
    # Script defined in ScriptData field
    SCRIPT_TYPE_INTERNAL_SCRIPT="internal_script"

    schema = FormAdapterSchema.copy() + Schema((
        StringField('ProxyRole',
            searchable=0,
            required=1,
            default="none",
            read_permission=ModifyPortalContent,
            write_permission=EDIT_PYTHON_PERMISSION,
            vocabulary="getProxyRoleChoices",
            widget=SelectionWidget(
                label='Proxy role',
                description = """ Role under which to run the script. """,
                label_msgid = "label_script_proxy",
                i18n_domain = "ploneformgen",
                description_msgid = "help_script_proxy",
                ),
            ),            
     
        PythonField('ScriptBody',
            searchable=0,
            required=0,
            default=default_script,
            read_permission=ModifyPortalContent,
            write_permission=EDIT_PYTHON_PERMISSION,
            widget=TextAreaWidget(
                label='Script body',
                rows=10,
                visible={'view': 'invisible','edit': 'visible'},    
                description = """ Write your script here. """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_script_body",
                description_msgid = "help_script_body"
                ),
            ),            
    ))
    if not HAS_PLONE30:
        finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)

    meta_type = portal_type = 'FormCustomScriptAdapter'
    archetype_name = 'Custom Script Adapter'

    content_icon   = 'scriptaction.gif'

    #immediate_view = 'fg_savedata_view'
    #default_view   = 'fg_savedata_view'

    security = ClassSecurityInfo()
    
    def __init__(self, oid, **kwargs):
        """ initialize class """

        FormActionAdapter.__init__(self, oid, **kwargs)
        
        # for the convenience of scripters operating
        # in a restricted python environment,
        # let's store a reference to FORM_ERROR_MARKER
        # on the object, so it'll be available
        # as an attribute of context.
        self.FORM_ERROR_MARKER = config.FORM_ERROR_MARKER
    
    def updateScript(self, body, role):
        """ Regenerate Python script object 
        
        Sync set of script source code, proxy role and 
        creation of Python Script object.
        
        """         
        bodyField = self.schema["ScriptBody"]    
        proxyField = self.schema["ProxyRole"]    
        script = PythonScript(self.title_or_id())
        script = script.__of__(self)
        
        # Force proxy role     
        if role != "none":
            script.manage_proxy((role,))
        
        script.ZPythonScript_edit("fields, ploneformgen, request", body)            
                
        PythonField.set(bodyField, self, script)        
        StringField.set(proxyField, self, role)
            
    def setScriptBody(self, body):
        """ Make PythonScript construction to take parameters """    
        proxy = self.getProxyRole()        
        self.updateScript(body, proxy)
                
    def setProxyRole(self, role):
        sourceCode = self.getRawScriptBody()
        self.updateScript(sourceCode, role)
        
    def getProxyRoleChoices(self):
        """ Get proxy role choices"""
        
        # XXX TODO: use real role list
        return DisplayList(
           (("none", "No proxy role"),
            ("Manager", "Manager"),
            ))
                
    def onSuccess(self, fields, REQUEST=None):
        """ Executes the custom script
        
        """
                        
        # use PloneFormGen object as a context
        form = aq_parent(self)       
        
        if REQUEST != None:
            resultData = self.sanifyFields(REQUEST.form) 
        else:
            resultData = {}
                        
        return self.executeCustomScript(resultData, form, REQUEST)
        
    def checkWarningsAndErrors(self):
        """ Raise exception if there has been bad things with the script compiling """
        field = self.schema["ScriptBody"]      
                  
        script = ObjectField.get(field, self)
        
        if len(script.warnings) > 0:
            logger.warn("Python script " + self.title_or_id() + " has warning:" + str(script.warnings))
        
        if len(script.errors) > 0:
            logger.error("Python script "  + self.title_or_id() +  " has errors: " + str(script.errors))
            raise ValueError, "Python script "  + self.title_or_id() + " has errors: " + str(script.errors)
    
    def executeCustomScript(self, result, form, req):
        """ Execute in-place script  
        
        @param result Extracted fields from REQUEST.form
        @param form PloneFormGen object
        """        
        field = self.schema["ScriptBody"] 
        # Now pass through PythonField/PythonScript abstraction 
        # to access bad things (tm)
        # otherwise there are silent failures
        script = ObjectField.get(field, self)
                
        logger.debug("Executing Custom Script Adapter " + self.title_or_id() + " fields:" + str(result))
        
        self.checkWarningsAndErrors()        
                    
        response = script(result, form, req)    
        return response
    
    def sanifyFields(self, form):
        """ Makes request.form fields accessible in a script 
        
        Avoid Unauthorized exceptions since REQUEST.form is inaccesible
        """
        result = {}
        for field in form:
            result[field] = form[field]
        return result
        
registerATCT(FormCustomScriptAdapter, PROJECTNAME)



from Products.PloneFormGen import HAS_PLONE30
from Products.PloneFormGen.config import *

from Products.CMFCore.utils import getToolByName

from StringIO import StringIO

def install(self):
    """ BBB: Make for a pleasant installation experience in 2.5.x.
        To be removed when eliminating support for < 3.x.
    """
    out = StringIO()
    
    # We install our product by running a GS profile.  We use the old-style Install.py module 
    # so that our product works w/ the Quick Installer in Plone 2.5.x
    print >> out, "Installing PloneFormGen"
    setup_tool = getToolByName(self, 'portal_setup')
    if HAS_PLONE30:
        setup_tool.runAllImportStepsFromProfile(
                "profile-Products.PloneFormGen:default",
                purge_old=False)
    else:
        # BBB: remove conditional once PFG no longer supports 2.5.x
        old_context = setup_tool.getImportContextID()
        
        # run the standard install process
        setup_tool.setImportContext('profile-Products.PloneFormGen:default')
        setup_tool.runAllImportSteps()
        
        # BBB: make 2.5.x specific overrides
        setup_tool.setImportContext('profile-Products.PloneFormGen:typeoverrides25x')
        setup_tool.runAllImportSteps()
        
        setup_tool.setImportContext(old_context)
    


def removeSkinLayer(self, layer):
    """ Remove a skin layer from all skinpaths """

    skinstool = getToolByName(self, 'portal_skins')
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName)
        path = [i.strip() for i in  path.split(',')]
        if layer in path:
            path.remove(layer)
            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)

def uninstall_tool(self, out):
    try:
        self.manage_delObjects(['formgen_tool'])
    except:
        pass
    else:
        print >>out, "PloneFormGen tool removed"


def uninstall(self):
    out = StringIO()

    uninstall_tool(self, out)
    
    # remove FormFolder from kupu's linkable types
    kupuTool = getToolByName(self, 'kupu_library_tool', None)
    typesTool = getToolByName(self, 'portal_types')
    if kupuTool is not None:
        linkable = list(kupuTool.getPortalTypesForResourceType('linkable'))

        # prune linkable to really existing types
        valid_types = dict([ (t.id, 1) for t in typesTool.listTypeInfo()])
        linkable = [pt for pt in linkable if pt in valid_types]

        if 'FormFolder' in linkable:
            linkable.remove('FormFolder')
        kupuTool.updateResourceTypes(({'resource_type' : 'linkable',
                                       'old_type'      : 'linkable',
                                       'portal_types'  :  linkable},))
        print >> out, "Removed FormFolder from kupu's linkable types"


    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')

    # remove the field, thanks and adapter types from types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in fieldTypes + adapterTypes + thanksTypes, fieldsetTypes:
        if f in typesNotListed:
            typesNotListed.remove(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Removed form fields & adapters from metaTypesNotToList"

    # Remove the field, thanks and adapter types from types_not_searched
    typesNotSearched = list(siteProperties.getProperty('types_not_searched'))
    for f in fieldTypes + adapterTypes + thanksTypes + fieldsetTypes:
        if f in typesNotSearched:
            typesNotSearched.remove(f)
    siteProperties.manage_changeProperties(types_not_searched = typesNotSearched)
    print >> out, "Removed form fields & adapters from types_not_searched"

    # Remove from default_page_types
    defaultPageTypes = list(siteProperties.getProperty('default_page_types'))
    if 'FormFolder' in defaultPageTypes:
        defaultPageTypes.remove('FormFolder')
        siteProperties.manage_changeProperties(default_page_types = defaultPageTypes)
    print >> out, "Removed FormFolder from default_page_types"

    # Remove from use_folder_tabs 
    use_folder_tabs = list(siteProperties.getProperty('use_folder_tabs'))
    if 'FormFolder' in use_folder_tabs:
        use_folder_tabs.remove('FormFolder')
    if 'FieldsetFolder' in use_folder_tabs:
        use_folder_tabs.remove('FieldsetFolder')
    siteProperties.manage_changeProperties(use_folder_tabs = use_folder_tabs)
    print >> out, "Removed FormFolder and FieldsetFolder from use_folder_tabs"

    # Remove from typesLinkToFolderContentsInFC 
    typesLinkToFolderContentsInFC = list(siteProperties.getProperty('typesLinkToFolderContentsInFC'))
    if 'FormFolder' in typesLinkToFolderContentsInFC:
        typesLinkToFolderContentsInFC.remove('FormFolder')
    if 'FieldsetFolder' in typesLinkToFolderContentsInFC:
        typesLinkToFolderContentsInFC.remove('FieldsetFolder')
    siteProperties.manage_changeProperties(typesLinkToFolderContentsInFC = typesLinkToFolderContentsInFC)
    print >> out, "Removed FormFolder and FieldsetFolder from typesLinkToFolderContentsInFC"

    # Remove skin directory from skin selections
    removeSkinLayer(self, 'PloneFormGen')
    print >> out, "Removed PloneFormGen layers from all skin selections"

    if HAS_PLONE30:
        kssRegTool = getToolByName(self, 'portal_kss', None)
        kssRegTool.manage_removeKineticStylesheet('ploneformgen.kss')
        print >> out, "Removed ploneformgen.kss from kss registry"

    print >> out, """\nNOTE: portal_properties/ploneformgen_properties
    was left in place so that you may reinstall without losing site
    configuration data. It's harmless, but feel free to delete it if
    you're sure you don't need any pfg site configuration changes."""

    print >> out, "\nSuccessfully uninstalled %s." % PROJECTNAME
    return out.getvalue()



from Products.validation import validation, interfaces

class ExRangeValidator:
    """ Validates whether or not a numeric value is within a range.
        minval and maxval *inclusive* may come from initialization,
        from the kwargs in a call or from field attributes.
    """


    __implements__ = (interfaces.ivalidator,)

    name = 'ExRangeValidator'

    def __init__(self, name, minval=0.0, maxval=0.0, title='', description=''):
        self.name = name
        self.minval = minval
        self.maxval = maxval
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):

        # import pdb; pdb.set_trace()
            
        field    = kwargs.get('field', None)

        # get maxval
        if kwargs.has_key('maxval'):
            maxval = kwargs.get('maxval')
        elif hasattr(field, 'maxval'):
            maxval = float(field.maxval)
        else:
            # set to given default value (default defaults to 0)
            maxval = self.maxval

        # get minval
        if kwargs.has_key('minval'):
            minval = kwargs.get('minval')
        elif hasattr(field, 'minval'):
            minval = float(field.minval)
        else:
            # set to given default value (default defaults to 0)
            minval = self.minval

        if (minval > maxval):
            return ("Validation failed(%(name)s): minval (%(min)s) > maxval (%(max)s)" %
                { 'name' : self.name, 'min' : minval, 'max' : maxval,})

        try:
            nval = float(value)
        except ValueError:
            return ("Validation failed(%(name)s): could not convert '%(value)s' to number" %
                    { 'name' : self.name, 'value': value})
                    
        if minval <= nval <= maxval:
            return 1

        if minval > nval:
            return ("Validation failed(%(name)s): '%(value)s' is too small. Must be at least %(min)s." %
                    { 'name' : self.name, 'value': value, 'min' : minval,})
        else:
            return ("Validation failed(%(name)s): '%(value)s' is too large. Must be no greater than %(max)s." %
                    { 'name' : self.name, 'value': value, 'max' : maxval,})

validation.register(ExRangeValidator('inExNumericRange'))



#
# Test PloneFormGen initialisation and set-up
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PloneFormGen.tests import pfgtc

from Products.PloneFormGen.content import validationMessages
from Products.validation import validation

        
class TestCustomValidators(pfgtc.PloneFormGenTestCase):
    """ test our validators """

    def test_inExNumericRange(self):
        v = validation.validatorFor('inExNumericRange')
        self.failUnlessEqual(v(10, minval=1, maxval=20), 1)
        self.failUnlessEqual(v('10', minval=1, maxval=20), 1)
        self.failUnlessEqual(v('1', minval=1, maxval=20), 1)
        self.failUnlessEqual(v('20', minval=1, maxval=20), 1)
        self.failIfEqual(v(0, minval=1, maxval=5), 1)
        self.failIfEqual(v(6, minval=1, maxval=5), 1)
        self.failIfEqual(v(4, minval=5, maxval=3), 1)

    def test_isNotTooLong(self):
        v = validation.validatorFor('isNotTooLong')
        self.failUnlessEqual(v('', maxlength=20), 1)
        self.failUnlessEqual(v('1234567890', maxlength=20), 1)
        self.failUnlessEqual(v('1234567890', maxlength=10), 1)
        self.failUnlessEqual(v('1234567890', maxlength=0), 1)
        self.failIfEqual(v('1234567890', maxlength=9), 1)
        self.failIfEqual(v('1234567890', maxlength=1), 1)

    def test_isChecked(self):
        v = validation.validatorFor('isChecked')
        self.failUnlessEqual(v('1'), 1)
        self.failIfEqual(v('0'), 1)

    def test_isUnchecked(self):
        v = validation.validatorFor('isUnchecked')
        self.failUnlessEqual(v('0'), 1)
        self.failIfEqual(v('1'), 1)

    def test_isNotLinkSpam(self):
        v = validation.validatorFor('isNotLinkSpam')
        good = """I am link free and proud of it"""
        bad1 = """<a href="mylink">Bad.</a>"""
        bad2 = """http://bad.com"""
        bad3 = """www.Bad.com"""
        bad = (bad1,bad2,bad3)
        class Mock(object):
            validate_no_link_spam = 1
        mock = Mock()
        kw = {'field':mock}
        self.failUnlessEqual(v(good, **kw), 1)
        for b in bad:
            self.failIfEqual(v(b, **kw), 1,
                             '"%s" should be considered a link.' % b)


    def test_isNotTooLong(self):
        v = validation.validatorFor('isNotTooLong')
        v.maxlength = 10
        self.failUnlessEqual(v('abc'), 1)
        self.failIfEqual(v('abcdefghijklmnopqrstuvwxyz'), 1)

        # there was a bug where widget.maxlength could possibly be defined as
        # '' which means calling int(widget.maxlength) would fail

        class Mock(object):
            pass
        field = Mock()
        field.widget = Mock()
        field.widget.maxlength = ''
        
        self.failUnlessEqual(v('abc', field=field), 1)

    def test_isEmail(self):
        v = validation.validatorFor('isEmail')
        self.failUnlessEqual(v('hi@there.com'), 1)
        self.failUnlessEqual(v('one@u.washington.edu'), 1)
        self.failIfEqual(v('@there.com'), 1)

    def test_isCommaSeparatedEmails(self):
        v = validation.validatorFor('pfgv_isCommaSeparatedEmails')
        self.failUnlessEqual(v('hi@there.com,another@two.com'), 1)
        self.failUnlessEqual(v('one@u.washington.edu,  two@u.washington.edu'), 1)
        self.failIfEqual(v('abc@plone.org; xyz@plone.org'), 1)


class TestCustomValidatorMessages(pfgtc.PloneFormGenTestCase):
    """ Test friendlier validation framework """
    
    def test_messageMassage(self):
    
        s = "Validation failed(isUnixLikeName): something is not a valid identifier."
        self.failUnlessEqual(validationMessages.cleanupMessage(s, self, self), u'pfg_isUnixLikeName')

        s = "Something is required, please correct."
        self.failUnlessEqual(validationMessages.cleanupMessage(s, self, self), u'pfg_isRequired')

        s = "Validation failed(isNotTooLong): 'something' is too long. Must be no longer than some characters."
        response = validationMessages.cleanupMessage(s, self, self)
        self.failUnlessEqual(response, u'pfg_too_long')


    def test_stringValidators(self):
        """ Test string validation
        """

        from Products.validation.exceptions import UnknowValidatorError
        from Products.validation import validation as v
        
        self.assertRaises(UnknowValidatorError, v.validate, 'noValidator', 'test')
            
        self.failIfEqual( v.validate('pfgv_isEmail', 'test'), 1 )

        self.failUnlessEqual( v.validate('pfgv_isEmail', 'test@test.com'), 1 )

        self.failUnlessEqual( v.validate('pfgv_isZipCode', '12345-1234'), 1 )


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCustomValidators))
    suite.addTest(makeSuite(TestCustomValidatorMessages))
    return suite



""" 
    
    Unit test for PloneFormGen custom scripts
    
    Copyright 2006 Red Innovation http://www.redinnovation.com
    
"""
__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'plaintext'

import os, sys


if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Globals import InitializeClass
from AccessControl import Unauthorized    
from AccessControl import ClassSecurityInfo
from Products.PythonScripts.PythonScript import PythonScript, manage_addPythonScript

from zExceptions import Forbidden

from Products.PloneFormGen.tests import pfgtc

from Products.CMFCore.utils import getToolByName
from Products.CMFCore import permissions

test_script="""
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, ploneformgen, request
##title=Succesfully working script
##

from Products.CMFCore.utils import getToolByName

assert fields["test_field"] == "123" 
return "foo"
"""

bad_parameters_script="""
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=
##title=
##

return 'foo'
"""

syntax_error_script="""
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, ploneformgen, request
##title=
##
if:
return asdfaf
"""

runtime_error_script="""
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, ploneformgen
##title=
##
return "asdfaf" + 1
"""

security_script="""
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, ploneformgen, request
##title=Script needing special privileges
##

from Products.CMFCore.utils import getToolByName

portal_url = getToolByName(context, "portal_url")
portal = portal_url.getPortalObject()

# Try set left_slots
portal.manage_addProperty('foo', ['foo'], 'lines')
"""

proxied_script="""
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, ploneformgen, request
##title=Script needing special privileges
##

# Should raise Unauthorized
return request.fooProtected()
"""

default_params_script = \
"""
fields
ploneformgen
request
"""

class FakeRequest(dict):
    
    def __init__(self, **kwargs):
        self.form = kwargs

class SecureFakeRequest(dict):
    
    security = ClassSecurityInfo()
        
    def __init__(self, **kwargs):
        self.form = kwargs
            
    security.declareProtected(permissions.ManagePortal,
                              'fooProtected')
    def fooProtected(self):
        """ Only manager can access this """
        return "foo"
        
InitializeClass(SecureFakeRequest)
        
        
class TestCustomScript(pfgtc.PloneFormGenTestCase):
    """ Test FormCustomScriptAdapter functionality in PloneFormGen """
    
    def afterSetUp(self):
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        
        self.loginAsPortalOwner()
                    
        self.folder.invokeFactory('FormFolder', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.invokeFactory('FormStringField', 'test_field')
        
        
    def createScript(self):
        """ Creates FormCustomScript object """
        # 1. Create custom script adapter in the form folder
        self.ff1.invokeFactory('FormCustomScriptAdapter', 'adapter')

        # 2. Check that creation succeeded
        self.failUnless('adapter' in self.ff1.objectIds())
        adapter = self.ff1.adapter
        
        # 3. Set action adapter
        self.ff1.setActionAdapter( ('adapter',) )
        self.assertEqual(self.ff1.actionAdapter, ('adapter',))       
    
#    def testScriptTypes(self):    
#        """ Check DisplayList doesn't fire exceptions """ 
#        self.createScript()        
#        adapter = self.ff1.adapter        
#        adapter.getScriptTypeChoices()
        
    def testSuccess(self): 
        """ Succesful script execution
        
        Creates a script, some form content,
        executes form handling.        
        """        
    
        self.createScript()
        
        adapter = self.ff1.adapter
        # 4. Set script data
        adapter.setScriptBody(test_script)                
        
        req = FakeRequest(test_field="123")
                        
        errors = adapter.validate()      
        assert len(errors) == 0, "Had errors:" + str(errors)          

        # Execute script
        # XXX TODO: we don't want to rely on return value from onSuccess
        reply = adapter.onSuccess(self.ff1, req) 
        assert reply == "foo", "Script returned:" + str(reply)
        
    def testRunTimeError(self):
        """ Script has run-time error """
        self.createScript()

        adapter = self.ff1.adapter
        # 4. Set script data
        adapter.setScriptBody(runtime_error_script)                
        
        errors = adapter.validate()      
        assert len(errors) == 0, "Had errors:" + str(errors)          

        # Execute script
        throwed = False
        try:
            reply = adapter.onSuccess([])
        except TypeError, e:
            reply = None
            throwed = True
                        
        assert throwed, "Bad script didn't throw run-time exception, got " + str(reply)
        assert reply == None        
        
    def testSyntaxError(self):
        """ Script has syntax errors 
        
        TODO: Syntax errors are not returned in validation?
        """
                        
        # Note: this test logs an error message; it does not indicate test failure

        self.createScript()
        
        adapter = self.ff1.adapter
        # 4. Set script data
        adapter.setScriptBody(syntax_error_script)                
        
        # Execute script
        throwed = False  
        try:
            adapter.onSuccess([])        
        except ValueError, e:
            throwed = True
            
        assert throwed, "Bad script didn't throw run-time exception"
 
        
    def testBadParameters(self):
        """ Invalid number of script parameters """

        self.createScript()
        
        adapter = self.ff1.adapter
        # 4. Set script data
        adapter.setScriptBody(bad_parameters_script)                
        
        errors = adapter.validate()      
        assert len(errors) == 0, "Had errors:" + str(errors)          

        # Execute script
        throwed = False
        try:
            adapter.onSuccess([])
        except TypeError:
            throwed=True
        assert throwed, "Invalid parameters failed silently"
        

    def testDefaultParameters(self):
        """ Test to make sure the documented parameters are available """

        self.createScript()
        
        adapter = self.ff1.adapter
        adapter.setScriptBody(default_params_script)
        
        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )
        

#    def testSecurity(self):
#        """ Script needing proxy role
#        
#        TODO: Why no security exceptions are raised?
#        """
#        #self.createScript()
#        
#        #adapter = self.ff1.adapter
#        # 4. Set script data
#        #adapter.setScriptBody(security_script)                
#        
#        #errors = adapter.validate()      
#        #assert len(errors) == 0, "Had errors:" + str(errors)          
#
#        # Execute script
#        #throwed = False
#        
#        #adapter.onSuccess([])
#
#        #if hasattr(self.portal, "foo"):
#        #    assert "Script executed under full priviledges"
#        
#        #self.failUnless(throwed==True, "Bypassed security, baaad!")
#        
#        pass


    def testSetProxyRole(self):
        """ Exercise setProxyRole """

        self.createScript()
        adapter = self.ff1.adapter

        adapter.setProxyRole('Manager')
        adapter.setProxyRole('none')
        
        self.assertRaises(Forbidden, adapter.setProxyRole, 'bogus')
        

# XXX TODO: We need to find another way to test this.    
#    def testProxyRole(self):
#        """ Test seeing how setting proxy role affects unauthorized exception """
#        
#        # TODO: Zope security system kills me
#        
#        #self.createScript()
#        
#        #adapter = self.ff1.adapter
#        # 4. Set script data
#        #adapter.setScriptBody(proxied_script)                
#        
#        #req = SecureFakeRequest(test_field="123")
#        #req = req.__of__(self.portal)
#                        
#        #errors = adapter.validate()      
#        #assert len(errors) == 0, "Had errors:" + str(errors)          
#        
#        # Execute script
#        #throwed = False
#        #try:
#        #    adapter.onSuccess([], req)
#        #except Unauthorized:
#        #    throwed=True
#        
#        #assert throwed, "No Unauthorized was raised"
#        
#        #self.loginAsPortalOwner()      
#        #self.setRoles(["Manager", "Owner"])  
#        #adapter.setProxyRole("Manager")
#        #self.logout()
#        
#        assert adapter.onSuccess([], req) == "123"
        
#    def testSkinsScript(self):
#        """ Test executing script from portal_skins """
#        portal_skins = self.portal.portal_skins
#        manage_addPythonScript(portal_skins.custom, "test_skins_script")
#        test_skins_script = portal_skins.custom.test_skins_script
#        
#        test_skins_script.ZPythonScript_edit("", test_script)
#        self._refreshSkinData()
#        
#
#        portal_skins.custom.test_skins_script({"test_field" : "123"}, "foo", None)  
#        # Do a dummy test call
#        self.portal.test_skins_script({"test_field" : "123"}, "foo", None)
#      
#        self.createScript()
#        adapter = self.ff1.adapter
#        adapter.setScriptType("skins_script")
#        adapter.setScriptName("test_skins_script")
#        
#        errors = adapter.validate()      
#        assert len(errors) == 0, "Had errors:" + str(errors)      
#   
#        # Execute script
#        req = FakeRequest(test_field="123")        
#        reply = adapter.onSuccess([], req) 
#        assert reply == "foo", "Script returned:" + str(reply)     
        
     
if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    from Products.PloneTestCase.layer import ZCMLLayer
    suite = TestSuite()
    suite.layer = ZCMLLayer
    suite.addTest(makeSuite(TestCustomScript))
    return suite



from zope.component import queryMultiAdapter

from Products.CMFCore.utils import getToolByName

from Products.PloneFormGen.config import PROPERTY_SHEET_NAME, \
    DEFAULT_MAILTEMPLATE_BODY, EXTRA_ALLOWED

from Products.PloneFormGen.interfaces.field import IPloneFormGenField
from Products.PloneFormGen.interfaces.actionAdapter import \
  IPloneFormGenActionAdapter
from Products.PloneFormGen.interfaces.fieldset import \
  IPloneFormGenFieldset
from Products.PloneFormGen.interfaces.thanksPage import \
  IPloneFormGenThanksPage
  
  
def update_kupu_resources(out, site):
    """ At the time of this writing, kupu's GS export/import
        handling is impractical.  We manage our interactions
        via kupu's arcane api's in the following.
    """
    # Add FormFolder to kupu's linkable types

    typesTool = getToolByName(site, 'portal_types')
    kupuTool = getToolByName(site, 'kupu_library_tool', None)
    if kupuTool is not None:
        linkable = list(kupuTool.getPortalTypesForResourceType('linkable'))
        
        if 'FormFolder' not in linkable:
            # kupu's resource list can accumulate old, no longer valid types;
            # it will throw an exception if we try to resave them.
            # So, let's clean the list.
            valid_types = dict([ (t.id, 1) for t in typesTool.listTypeInfo()])
            linkable = [pt for pt in linkable if pt in valid_types]

            linkable.append('FormFolder')
            kupuTool.updateResourceTypes(({'resource_type' : 'linkable',
                                           'old_type'      : 'linkable',
                                           'portal_types'  :  linkable},))

def safe_add_purgeable_properties(out, site):
    """ In order to avoid a possible "feature" regression and
        to keep test case testModificationsToPropSheetNotOverwritten in 
        a passing state, we need to do a check before property add
        of all non-lines properties. This per my reading of GS' PropertiesXMLAdapter's 
        _initProperties implementation, which appears to only merge properties of
        type tuple or list.
    """
    ppTool = getToolByName(site, 'portal_properties')
    propSheet = getattr(ppTool, PROPERTY_SHEET_NAME)
    if not propSheet.hasProperty('mail_template'):
        propSheet.manage_addProperty('mail_template', DEFAULT_MAILTEMPLATE_BODY, 'text')    
    if not propSheet.hasProperty('mail_body_type'):
        propSheet.manage_addProperty('mail_body_type', 'html', 'string')    
    if not propSheet.hasProperty('mail_recipient_email'):
        propSheet.manage_addProperty('mail_recipient_email', '', 'string')    
    if not propSheet.hasProperty('mail_recipient_name'):
        propSheet.manage_addProperty('mail_recipient_name', '', 'string')
    if not propSheet.hasProperty('csv_delimiter'):
        propSheet.manage_addProperty('csv_delimiter', ',', 'string')        
    

def setAllowed(pti, types):
    """
    Add types to allowed in a portal type
    """
    
    changed = False
    
    newType = 'FormCaptchaField'
    if haveCaptcha:
        if newType not in myTypes:
            myTypes.append(newType)
            changed = True
    else:
        if newType in myTypes:
            myTypes.remove(newType)
            changed = True
    if changed:
        ptType.manage_changeProperties(allowed_content_types = myTypes)
    

def importVarious(context):
    """
    Final PloneFormGen import steps.
    """
    # Only run step if a flag file is present (e.g. not an extension profile)
    if context.readDataFile('ploneformgen-various.txt') is None:
        return
    out = []
    site = context.getSite()
    update_kupu_resources(out, site)
    safe_add_purgeable_properties(out, site)

    ##############
    # set allowed types for insertion in form folder, fieldsets

    # get a list of installed meta types
    ptt = getToolByName(site, 'portal_types')
    att = getToolByName(site, 'archetype_tool')
    metatypes = [ct.content_meta_type for ct in ptt.listTypeInfo()]
    installed_types = [ttype for ttype in att.listTypes() if ttype.meta_type in metatypes]

    # look for PFG fields, fieldsets, thankers and adapters by interface
    fields = [ttype.meta_type for ttype in installed_types
              if IPloneFormGenField.implementedBy(ttype)]
    adapters = [ttype.meta_type for ttype in installed_types
                if IPloneFormGenActionAdapter.implementedBy(ttype)]
    fieldsets = [ttype.meta_type for ttype in installed_types
                 if IPloneFormGenFieldset.implementedBy(ttype)]
    thankers = [ttype.meta_type for ttype in installed_types
                if IPloneFormGenThanksPage.implementedBy(ttype)]

    # can't do captcha field without captcha support, so
    # look for a view named captcha.
    # if there isn't one, remove the field
    if ('FormCaptchaField' in fields) and \
       not queryMultiAdapter((site, site.REQUEST), name='captcha'):
        fields.remove('FormCaptchaField')

    # now, use our hard-won type lists to set allowed types
    ptt.getTypeInfo('FormFolder').manage_changeProperties(
      allowed_content_types = fields + adapters + fieldsets + thankers + \
        EXTRA_ALLOWED
      )
    for fs in fieldsets:
        ptt.getTypeInfo(fs).manage_changeProperties(
          allowed_content_types = fields
          )



""" FormGenTool manages site-wide PloneFormGen settings """ 

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from types import StringTypes

from AccessControl import ClassSecurityInfo
from AccessControl.PermissionRole import rolesForPermissionOn
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem

from Products.CMFCore.permissions import View, ManagePortal, ModifyPortalContent
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFPlone.utils import safe_hasattr

from Products.Archetypes.utils import DisplayList

from Products.validation import validation
from Products.validation.validators.RegexValidator import RegexValidator

from Globals import InitializeClass 

from Products.PloneFormGen import config

from Products.PloneFormGen import PloneFormGenMessageFactory as _


class FormGenTool(UniqueObject, SimpleItem): 
    """ FormGenTool manages site-wide PloneFormGen settings """ 

    id = 'formgen_tool'
    meta_type= 'PloneFormGen Tool' 
    title = 'PloneFormGen Settings'
    plone_tool = True
    security = ClassSecurityInfo()
    

    def __init__(self):
        self._initStringValidators()

    
    security.declarePrivate('_initStringValidators')
    def _initStringValidators(self):
        """ Initialize string validators from config
        """

        self.stringValidators = {}
        self.stringValidatorsDL = DisplayList()
        self.stringValidatorsDL.add('vocabulary_none_text', u'None')

        for kwa in config.stringValidators:
            id = kwa['id']    

            title = kwa.get('title', id)
            i18nid = kwa.get('i18nid', title)

            errmsg = kwa.get('errmsg', 'Validation failed: %s' % id)
            errid = kwa.get('errid', errmsg)
            errmsg = _(errid, errmsg)

            validatorId = 'pfgv_%s' % id
            self.stringValidators[id] = {
                'title'  : title,
                'i18nid' : i18nid,
                'errmsg' : errmsg,
                'errid'  : errid,
                # 'revalid': revalid,
                'id'     : validatorId,
                }
            
            self.stringValidatorsDL.add( id, title, msgid=i18nid )


    def getStringValidatorsDL(self):
        """ return a display list of string validators
        """

        if not getattr(self, 'stringValidators', False):
            # on-demand migration from PFG < 1.2
            self._initStringValidators()

        # self._initStringValidators()

        return self.stringValidatorsDL


    security.declarePrivate('getFromPropSheet')
    def getFromPropSheet(self, propid, default):
        """ find a property value in the property sheet
            with a fallback """
    
        res = default
        ppTool = getToolByName(self, 'portal_properties')       
        psheet = getattr(ppTool, config.PROPERTY_SHEET_NAME, None)
        if psheet is not None:
            res = psheet.getProperty(propid, res)
        return res


    security.declareProtected(ManagePortal, 'setDefault')
    def setDefault(self, propid, default):
        """ change a property in the portal_properties sheet """
        
        ppTool = getToolByName(self, 'portal_properties')       
        psheet = getattr(ppTool, config.PROPERTY_SHEET_NAME)
        psheet.manage_changeProperties( **{propid : default} )        

        
    security.declareProtected(ManagePortal, 'getPfgPermissions')
    def getPfgPermissions(self):
        """ get permissions in use by PFG """

        return self.getFromPropSheet('permissions_used', config.pfgPermitList)


    security.declareProtected(ModifyPortalContent, 'getDefaultMailTemplateBody')
    def getDefaultMailTemplateBody(self):
        """ get the site's default mail adapter mail body template """
        
        return self.getFromPropSheet('mail_template', config.DEFAULT_MAILTEMPLATE_BODY)


    security.declareProtected(ModifyPortalContent, 'getDefaultMailRecipient')
    def getDefaultMailRecipient(self):
        """ get the site's default mail adapter recipient """
        
        return self.getFromPropSheet('mail_recipient_email', '')


    security.declareProtected(ModifyPortalContent, 'getDefaultMailCC')
    def getDefaultMailCC(self):
        """ get the site's default mail cc """
        
        return self.getFromPropSheet('mail_cc_recipients', [])


    security.declareProtected(ModifyPortalContent, 'getDefaultMailBCC')
    def getDefaultMailBCC(self):
        """ get the site's default mail cc """
        
        return self.getFromPropSheet('mail_bcc_recipients', [])


    security.declareProtected(ModifyPortalContent, 'getDefaultMailRecipientName')
    def getDefaultMailRecipientName(self):
        """ get the site's default mail adapter recipient name """
        
        return self.getFromPropSheet('mail_recipient_name', '')


    security.declareProtected(ModifyPortalContent, 'getDefaultMailBodyType')
    def getDefaultMailBodyType(self):
        """ get the site's default mail adapter mail body template """
        
        return self.getFromPropSheet('mail_body_type', 'html')


    security.declareProtected(ModifyPortalContent, 'getCSVDelimiter')
    def getCSVDelimiter(self):
        """ get the site's default csv delimiter for data export """
        
        return self.getFromPropSheet('csv_delimiter', ',')


    security.declareProtected(ModifyPortalContent, 'getDefaultMailXInfo')
    def getDefaultMailXInfo(self):
        """ get the site's default mail adapter xinfo headers """
        
        return self.getFromPropSheet('mail_xinfo_headers', config.XINFO_DEFAULT)


    security.declareProtected(ModifyPortalContent, 'getDefaultMailAddHdrs')
    def getDefaultMailAddHdrs(self):
        """ get the site's default mail adapter additional headers """
        
        return self.getFromPropSheet('mail_add_headers', [])


    security.declareProtected(ManagePortal, 'rolesForPermission')
    def rolesForPermission(self, permit):
        """
         return list of roles for permission,
         ready to use in permissions configlet
        """

        portal=getToolByName(self, 'portal_url').getPortalObject()

        livePermits = rolesForPermissionOn(permit, portal)
        myPermits = list(self.getPfgPermissions())
        index = myPermits.index(permit)
        rpos = 0
        res = []
        for role in portal.rolesOfPermission(permit):
            name = role['name']
            if name not in ['Anonymous', 'Authenticated']:
                id = "p%dr%d" % (index, rpos)
                if name in livePermits:
                    checked = 'CHECKED'
                else:
                    checked = None
                res.append( {'label':name, 'id':id, 'checked':checked,} )
            rpos += 1
        return res


    security.declareProtected(ManagePortal, 'setRolePermits')
    def setRolePermits(self, REQUEST):
        """
        Set role/permissions on portal based on REQUEST.form.
        For use in configlet.
        """
        
        portal=getToolByName(self, 'portal_url').getPortalObject()
        permits = list(self.getPfgPermissions())

        # build list of roles selected in form
        for index in range(0, len(permits)):
            permit = permits[index]
            # look to see if there is a marker for permit
            # in the form. That makes sure we don't set unintended
            # permissions.
            if REQUEST.form.get(permit, '0') == '1':
                allRoles = portal.rolesOfPermission(permit)
                rpos = 0
                roles = []
                for rpos in range(0, len(allRoles)):
                    id = "p%dr%d" % (index, rpos)
                    if REQUEST.form.get(id, '0') == '1':
                        roles.append(allRoles[rpos]['name'])
                # set role permissions
                portal.manage_permission(permit, roles)            
    
        
InitializeClass(FormGenTool)

# since we're in a singleton's module anyway,
# this is a convenient place to register the
# custom string validators

def _registerStringValidators():

    for kwa in config.stringValidators:
        id = kwa['id']    

        errmsg = kwa.get('errmsg', 'Validation failed: %s' % id)
        errid = kwa.get('errid', errmsg)
        errmsg = _(errid, errmsg)

        # create a validator to match, register it.
        validatorId = 'pfgv_%s' % id
        revalid = RegexValidator(validatorId,
            kwa.get('regex', '.+'),
            ignore=kwa.get('ignore', ''),
            )
        validation.register(revalid)

_registerStringValidators()



import logging

from Acquisition import aq_parent, aq_inner
from zope.component import adapter
from zope.app.container.interfaces import IObjectAddedEvent

from Products.CMFPlone.interfaces import IFactoryTool

from Products.PloneFormGen import interfaces

@adapter(interfaces.IPloneFormGenActionAdapter, IObjectAddedEvent)
def form_adapter_pasted(form_adapter, event):
    """If an action adapter is pasted into the form, add it to the form's 
       list of active adapters. We only need to do anything if the action
       adapter isn't newly created in the portal_factory.
    """
    form_adapter = aq_inner(form_adapter)
    if IFactoryTool.providedBy(aq_parent(aq_parent(form_adapter))):
        return
        
    form = aq_parent(form_adapter)
    adapters = list(form.actionAdapter)
    if form_adapter.id not in adapters:
        adapters.append(form_adapter.id)
        form.setActionAdapter(adapters)



"""validationMessages -- cleanup Archetypes/validation messages"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

# Messages taken from godchap's gotcha-validation-i18n branch of
# archetypes validation

from Products.CMFCore.utils import getToolByName

import re
from types import StringTypes

verrorRE = re.compile("Validation failed\((.+)\):")
enrMessageRE = re.compile( "Validation failed\(inExNumericRange\): could not convert '(.+)' to number" )
enrSmallMessageRE = re.compile( "Validation failed\(inExNumericRange\): '(.+?)' is too small. Must be at least (.+)." )
enrLargeMessageRE = re.compile( "Validation failed\(inExNumericRange\): '(.+?)' is too large. Must be no greater than (.+)." )
mlTooLongRE = re.compile( "Validation failed\(isNotTooLong\): '.+?' is too long. Must be no longer than (.+) characters." )


from Products.PloneFormGen import PloneFormGenMessageFactory as _

# The commented out messages are now part of the general purpose RE validator.
# See config.py for initial values.
newMessages = {
    'isDecimal' :   _(u'pfg_isDecimal', u'This field requires a decimal value.'),
    'isInt' :       _(u'pfg_isInt', u'This field requires an integer value.'),
    'isPrintable' : _(u'pfg_isPrintable', u'This value contains unprintable characters.'),
    'isSSN' :       _(u'pfg_isSSN', u'This is not a well formed SSN.'),
    # 'isUSPhoneNumber' : _(u'pfg_isUSPhoneNumber', u'This is not a valid us phone number.'),
    # 'isInternationalPhoneNumber' : _(u'pfg_isInternationalPhoneNumber', u'This is not a valid international phone number.'),
    # 'isZipCode' :   _(u'pfg_isZipCode', u'This is not a valid zip code.'),
    # 'isURL' :       _(u'pfg_isURL', u'This is not a valid url.'),
    # 'isEmail' :     _(u'pfg_isEmail', u'This is not a valid email address.'),
    'isMailto' :    _(u'pfg_isMailto', u'This is not a valid mailto: url.'),
    'isUnixLikeName' : _(u'pfg_isUnixLikeName', u'This name is not a valid identifier.'),
    'isChecked' :   _(u'pfg_isChecked', u'This box must be checked.'),
    'isUnchecked' :   _(u'pfg_isUnchecked', u'This box must be unchecked.'),
    'isValidDate' :   _(u'pfg_isValidDate', u'The date entered was invalid.'),
    'isNotLinkSpam': _(u'pfg_isnotlinkspam', u'This text appears to contain links.'),
    }

newRequiredMessage = _(u'pfg_isRequired', u'This field is required.')

def cleanupMessage(original, context, instance):
    """ Where original is the message from the Archetypes validator,
        return an improved, translatable message if available.
    """

    if type(original) in StringTypes:
        if original.find('is required, please correct.') > 0:
            return newRequiredMessage
        else:
            mo = verrorRE.match(original)
            if mo:
                term = mo.groups()[0]

                if term.find('pfgv_') == 0:
                    # this is one of the customizable pfg tool validators
                    fgt = getToolByName(instance, 'formgen_tool')
                    sv = fgt.stringValidators.get(term[5:])
                    if sv:
                        # this is already a messagestr
                        return sv['errmsg']

                nm = newMessages.get(term)
                if nm:
                    # this is one of the simple messages for which we have a replacement
                    return nm

                if term == 'inExNumericRange':
                    mo = enrMessageRE.match(original)
                    if mo:
                        return _(u'pfg_not_number', u'Please enter a number here.')
                    mo = enrSmallMessageRE.match(original)
                    if mo:
                        groups = mo.groups()
                        return _(u'pfg_number_too_small',
                                 u"'${value}' is too small. Must be at least ${min}.",
                                 mapping={ 'value':groups[0], 'min':groups[1] })
                    mo = enrLargeMessageRE.match(original)
                    if mo:
                        groups = mo.groups()
                        return _(u'pfg_number_too_large',
                                 u"'${value}' is too large. Must be no greater than ${max}.",
                                 mapping={ 'value':groups[0], 'max':groups[1] })
                elif term == 'isNotTooLong':
                    mo = mlTooLongRE.match(original)
                    if mo:
                        groups = mo.groups()
                        return _(u'pfg_too_long',
                                 u"'Entry too long. It should be no more than ${max} characters.",
                                 mapping={ 'max':groups[0] })

    return original



"""Fieldset -- widgets for fieldset start / end"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget

class FieldsetStartWidget(TypesWidget):
    """
        This pseudo widget opens an XHTML fieldset.
    """

    _properties = TypesWidget._properties.copy()
    _properties.update({'macro' : 'widget_fieldset_start',
                        'show_legend' : True,
                        },)

# Register the widget with Archetypes
registerWidget(FieldsetStartWidget,
               title = 'Fieldset start widget',
               description= ('Renders XHTML fieldset open tag',),
               used_for = ('Products.PloneFormGen.contents.fieldset.FieldsetFolder',)
               )


class FieldsetEndWidget(TypesWidget):
    """
        This pseudo widget closes an XHTML fieldset.
    """

    _properties = TypesWidget._properties.copy()
    _properties.update({'macro' : 'widget_fieldset_end',
                        },)

# Register the widget with Archetypes
registerWidget(FieldsetEndWidget,
               title = 'Fieldset end widget',
               description= ('Renders XHTML fieldset end tag',),
               used_for = ('Products.PloneFormGen.contents.fieldset.FieldsetFolder',)
               )



import logging
from Products.PloneFormGen import HAS_PLONE30
from Products.PloneFormGen.config import PLONE_25_PUBLISHER_MONKEYPATCH

def patch_portlet_error_handling():
    """
    Add 'Retry' to the exceptions that the portlet machinery doesn't
    catch for displaying a "broken portlet" message.
    """
    logger = logging.getLogger('PloneFormGen')
    logger.info('Patching plone.app.portlets ColumnPortletManagerRenderer to not catch Retry exceptions')
    
    import sys
    from Acquisition import aq_acquire
    from ZODB.POSException import ConflictError
    from ZPublisher.Publish import Retry

    from plone.app.portlets.manager import ColumnPortletManagerRenderer, logger

    def safe_render(self, portlet_renderer):
        try:
            return portlet_renderer.render()
        except (ConflictError, Retry):
            raise
        except Exception:
            logger.exception('Error while rendering %r' % (self,))
            aq_acquire(self, 'error_log').raising(sys.exc_info())
            return self.error_message()

    ColumnPortletManagerRenderer.safe_render = safe_render


def patch_publisher_exception_handling():
    # # The following code is taken verbatim from Plone 3's plone.app.linkintegrity.
    # We need it for its modification of Retry exception handling.
    logger = logging.getLogger('PloneFormGen')
    logger.info('Patching Zope publisher to support retries')

    from zope.component import queryMultiAdapter
    from Zope2.App.startup import zpublisher_exception_hook
    from ZPublisher.Publish import Retry

    def zpublisher_exception_hook_wrapper(published, REQUEST, t, v, traceback):
        """ wrapper around the zope2 zpublisher's error hook """
        try:
            # if we got a retry exception, we just propagate it instead of
            # trying to log it (like FiveException does)
            if t is Retry:
                v.reraise()
            # first we try to find a view/adapter for the current exception and
            # let the original function try to handle the exception if we can't
            # find one...
            view = queryMultiAdapter((v, REQUEST), name='index.html', default=None)
            if view is None:
                zpublisher_exception_hook(published, REQUEST, t, v, traceback)
            else:
                # otherwise render the view and raise the rendered string like
                # raise_standardErrorMessage does...
                view = view.__of__(published)
                message = view()
                if isinstance(message, unicode):
                    message = message.encode('utf-8')
                raise t, message, traceback
        finally:
            traceback = None


    from ZPublisher.Publish import get_module_info
    def proxy_get_module_info(*args, **kwargs):
        results = list(get_module_info(*args, **kwargs))
        if results[5] is zpublisher_exception_hook:
            results[5] = zpublisher_exception_hook_wrapper
        return tuple(results)


    def installExceptionHook():
        import ZPublisher.Publish
        ZPublisher.Publish.get_module_info = proxy_get_module_info


    def retry(self):
        """ re-initialize a response object to be used in a retry attempt """
        # this implementation changes the original one so that the response
        # instance is reused instead of replaced with a new one (after a Retry
        # exception was raised);  this fixes a bug in zopedoctests' http()
        # function (Testing/ZopeTestCase/zopedoctest/functional.py:113);
        # the doctest code assumes that the HTTPResponse instance passed to
        # publish_module() (line 177) is used to handle to complete request, so
        # it can be used to get the status, headers etc later on (lines 183-186);
        # normally this is okay, but raising a Retry will create a new response
        # instance, which will then hold that data (relevant for evaluating the
        # doctest) while the original (passed in) instance is still empty...
        #
        # so to fix this (quickly) retry() now cleans up and returns itself:
        self.__init__(stdout=self.stdout, stderr=self.stderr)
        return self

    from ZPublisher.HTTPResponse import HTTPResponse
    HTTPResponse.retry = retry

    installExceptionHook()


if HAS_PLONE30:
    patch_portlet_error_handling()

if not HAS_PLONE30 and PLONE_25_PUBLISHER_MONKEYPATCH:
    patch_publisher_exception_handling()



import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PloneFormGen.tests import pfgtc

from ZPublisher.Publish import Retry

class FakeRequest(dict):

    def __init__(self, **kwargs):
        self.form = kwargs

class TestEmbedding(pfgtc.PloneFormGenTestCase):
    """ test embedding of a PFG in another template """

    def dummy_send( self, mfrom, mto, messageText ):
        self.mfrom = mfrom
        self.mto = mto
        self.messageText = messageText
        
    def LoadRequestForm(self, **kwargs):
        form = self.app.REQUEST.form
        form.clear()
        for key in kwargs.keys():
            form[key] = kwargs[key]
        return self.app.REQUEST

    def afterSetUp(self):
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        self.folder.invokeFactory('FormFolder', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.checkAuthenticator = False # no csrf protection
        self.mailhost = self.folder.MailHost
        self.mailhost._send = self.dummy_send
        self.ff1.mailer.setRecipient_email('mdummy@address.com')

    def test_embedded_form_renders(self):
        view = self.ff1.restrictedTraverse('@@embedded')
        res = view()
        
        # form renders
        self.failUnless('Your E-Mail Address' in res)
        
        # form action equals request URL
        self.failUnless('action="%s"' % self.app.REQUEST['URL'] in res)
        
        # no form prefix
        self.failUnless('name="form.submitted"' in res)
        
        # we can specify a form prefix
        view.setPrefix('mypfg')
        res = view()
        self.failUnless('name="mypfg.form.submitted"' in res)

    def test_embedded_form_validates(self):
        # fake an incomplete form submission
        self.LoadRequestForm(**{
            'mypfg.form.submitted': True,
            })

        # render the form
        view = self.ff1.restrictedTraverse('@@embedded')
        view.setPrefix('mypfg')
        res = view()

        # should stay on same page on errors, and show messages
        self.failUnless('This field is required.' in res)

    def test_doesnt_process_submission_of_other_form(self):
        # fake submission of a *different* form (note mismatch of form submission marker with prefix)
        self.LoadRequestForm(**{
            'form.submitted': True,
            })

        # let's preset a faux controller_state (as if from the other form)
        # to make sure it doesn't throw things off
        self.app.REQUEST.set('controller_state', 'foobar')

        # render the form
        view = self.ff1.restrictedTraverse('@@embedded')
        view.setPrefix('mypfg')
        res = view()

        # should be no validation errors
        self.failIf('This field is required.' in res)
        
        # (and request should still have the 'form.submitted' key)
        self.failUnless('form.submitted' in self.app.REQUEST.form)
        
        # (and the controller state should be untouched)
        self.assertEqual(self.app.REQUEST.get('controller_state'), 'foobar')
        
        # but if we remove the form prefix then it should process the form
        view.setPrefix('')
        res = view()
        self.failUnless('This field is required.' in res)

    def test_render_thank_you_on_success(self):
        self.LoadRequestForm(**{
            'form.submitted': True,
            'topic': 'monkeys',
            'comments': 'I am not a walnut.',
            'replyto': 'foobar@example.com'
            })

        # should raise a retry exception triggering a new publish attempt
        # with the new URL
        # XXX do a full publish for this test
        view = self.ff1.restrictedTraverse('@@embedded')
        self.assertRaises(Retry, view)


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestEmbedding))
    return suite



from zope.interface import Interface, Attribute

# This is just a marker inteface; PFG identifies adapters by asking
# contained objects if they provide this interface.
#
# Note that marking an object type with this interface 
# does not automatically put it on the "add" menu. That
# needs to be done via portal_types.

class IPloneFormGenActionAdapter(Interface):
    """actionAdapter interface
    """
    
    meta_type = Attribute("archetypes meta type")
    """
    Must match GS type declaration.
    """

    def onSuccess(fields, REQUEST=None):
        """
        Called by form to invoke custom success processing.
        return None (or don't use "return" at all) if processing is
        error-free.
        
        Return a dictionary like {'field_id':'Error Message'}
        and PFG will stop processing action adapters and
        return back to the form to display your error messages
        for the matching field(s).

        You may also use Products.PloneFormGen.config.FORM_ERROR_MARKER
        as a marker for a message to replace the top-of-the-form error
        message.

        For example, to set a message for the whole form, but not an
        individual field:

        {FORM_ERROR_MARKER:'Yuck! You will need to submit again.'}

        For both a field and form error:

        {FORM_ERROR_MARKER:'Yuck! You will need to submit again.',
         'field_id':'Error Message for field.'}
        
        Messages may be string types or zope.i18nmessageid objects.                
        """

        execCondition = Attribute("Execution Condition")
        """
        If this attribute exists, the form folder will
        use getRawExecCondition.
        """

        def getRawExecCondition():
            """
            Optional.
            
            Should return a TALES expession that will be evaluated
            in the context of the object to determine whether or not
            to execute an action adapter's onSuccess method.
            """


#
# Test PloneFormGen initialisation and set-up
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PloneFormGen.tests import pfgtc
from Products.PloneFormGen import HAS_PLONE30

from Products.CMFCore.utils import getToolByName

import Products

def getAddPermission(product, name):
    """ find the add permission for a meta_type """

    name = "%s: %s" % (product, name)
    for mt in Products.meta_types:
        if mt['name'] == name:
            return mt['permission']
    return ""

class TestInstallation(pfgtc.PloneFormGenTestCase):
    """Ensure product is properly installed"""

    def afterSetUp(self):
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        
        self.kupu         = self.portal.kupu_library_tool
        self.skins        = self.portal.portal_skins
        self.types        = self.portal.portal_types
        self.factory      = self.portal.portal_factory
        self.workflow     = self.portal.portal_workflow
        self.properties   = self.portal.portal_properties
        self.at_tool      = self.portal.archetype_tool
        self.controlpanel = self.portal.portal_controlpanel

        fieldTypes = [
            'FormSelectionField',
            'FormMultiSelectionField',
            'FormLabelField',
            'FormDateField',
            'FormLinesField',
            'FormIntegerField',
            'FormBooleanField',
            'FormPasswordField',
            'FormFixedPointField',
            'FormStringField',
            'FormTextField',
            'FormRichTextField',
            'FormRichLabelField',
            'FormFileField',
            'FormLikertField',
        ]
        if pfgtc.haveRecaptcha:
            fieldTypes.append('FormCaptchaField')
        self.fieldTypes = tuple(fieldTypes)
        self.adapterTypes = (
            'FormSaveDataAdapter',
            'FormMailerAdapter',
            'FormCustomScriptAdapter',
        )
        self.thanksTypes = (
            'FormThanksPage',
        )
        self.fieldsetTypes = (
            'FieldsetFolder',
        )
        self.metaTypes = ('FormFolder',) + self.fieldTypes + self.adapterTypes + self.thanksTypes + self.fieldsetTypes

    def testSkinLayersInstalled(self):
        self.failUnless('PloneFormGen' in self.skins.objectIds())

    def testSkinLayersInSkinPath(self):
        pfg_layers = self.skins['selections']
        
        for skin_name, path in pfg_layers.items():
            self.failUnless('PloneFormGen' in path.split(','))
    
    def testKssRegsitry(self):
        if 'portal_kss' in self.portal.objectIds():
            # confirm kinetic stylesheet registration
            for kss_id in ('ploneformgen.kss',):
                self.failUnless(kss_id in self.portal.portal_kss.getResourceIds(),
                    "The kss resource %s wasn't registered appropriately with the portal_kss registry")

    def testTypesInstalled(self):
        for t in self.metaTypes:
            self.failUnless(t in self.types.objectIds())
            
    def testTypeActions(self):
        if HAS_PLONE30:
            # hide properties/references tabs
            for typ in self.metaTypes:
                for act in self.types[typ].listActions():
                    if act.id in ['metadata', 'references']:
                        self.failIf(act.visible)
        else:
            # the 2.5.x way is to enable these tabs
            for typ in self.metaTypes:
                for act in self.types[typ].listActions():
                    if act.id in ['metadata', 'references']:
                        self.failUnless(act.visible)
    
    def testArchetypesToolCatalogRegistration(self):
        for t in self.metaTypes:
            self.assertEquals(1, len(self.at_tool.getCatalogsByType(t)))
            self.assertEquals('portal_catalog', self.at_tool.getCatalogsByType(t)[0].getId())
            
    def testControlPanelConfigletInstalled(self):
        self.failUnless('PloneFormGen' in [action.id for action in self.controlpanel.listActions()])
    
    def testAddPermissions(self):
        """ Test to make sure add permissions are as intended """
        
        ADD_CONTENT_PERMISSION = 'PloneFormGen: Add Content'
        CSA_ADD_CONTENT_PERMISSION = 'PloneFormGen: Add Custom Scripts'
        MA_ADD_CONTENT_PERMISSION = 'PloneFormGen: Add Mailers'
        SDA_ADD_CONTENT_PERMISSION = 'PloneFormGen: Add Data Savers'
      
        self.assertEqual( getAddPermission('PloneFormGen', 'Form Folder'), ADD_CONTENT_PERMISSION)
        self.assertEqual( getAddPermission('PloneFormGen', 'Mailer Adapter'), MA_ADD_CONTENT_PERMISSION)
        self.assertEqual( getAddPermission('PloneFormGen', 'Save Data Adapter'), SDA_ADD_CONTENT_PERMISSION)
        self.assertEqual( getAddPermission('PloneFormGen', 'Custom Script Adapter'), CSA_ADD_CONTENT_PERMISSION)
    
    def testActionsInstalled(self):
        self.setRoles(['Manager',])
        self.failUnless(self.portal.portal_actions.getActionInfo('object_buttons/export'))
        self.failUnless(self.portal.portal_actions.getActionInfo('object_buttons/import'))
    
    def testPortalFactorySetup(self):
        for f in self.metaTypes:
            self.failUnless(f in self.factory.getFactoryTypes())

    def testTypesNotSearched(self):
        types_not_searched = self.properties.site_properties.getProperty('types_not_searched')
        for f in self.fieldTypes + self.adapterTypes + self.thanksTypes + self.fieldsetTypes:
            self.failUnless(f in types_not_searched)

    def testTypesNotListed(self):
        metaTypesNotToList  = self.properties.navtree_properties.getProperty('metaTypesNotToList')
        for f in self.fieldTypes + self.adapterTypes + self.thanksTypes + self.fieldsetTypes:
            self.failUnless(f in metaTypesNotToList)

    def testFieldsHaveNoWorkflow(self):
        for f in self.fieldTypes + self.fieldsetTypes:
            self.assertEqual(self.workflow.getChainForPortalType(f), ())

    def testAdaptersHaveNoWorkflow(self):
        for f in self.adapterTypes:
            self.assertEqual(self.workflow.getChainForPortalType(f), ())

    def testThankspagessHaveNoWorkflow(self):
        for f in self.thanksTypes:
            self.assertEqual(self.workflow.getChainForPortalType(f), ())

    def testKupuResources(self):
        linkable = self.kupu.getPortalTypesForResourceType('linkable')
        self.failUnless('FormFolder' in linkable) # make sure we made it in ...
        self.failIf(len(linkable) <= 1) # without clobbering everything else

    def test_FormGenTool(self):
        self.failUnless( getToolByName(self.portal, 'formgen_tool')) 

    def test_PropSheetCreation(self):
        props = getattr(self.properties, 'ploneformgen_properties', None)
        self.failUnless( props )
        self.failUnless( props.hasProperty('permissions_used') )
        self.failUnless( props.hasProperty('mail_template') )
        self.failUnless( props.hasProperty('mail_body_type') )
        self.failUnless( props.hasProperty('mail_recipient_email') )
        self.failUnless( props.hasProperty('mail_cc_recipients') )
        self.failUnless( props.hasProperty('mail_bcc_recipients') )
        self.failUnless( props.hasProperty('mail_xinfo_headers') )
        self.failUnless( props.hasProperty('mail_add_headers') )
        self.failUnless( props.hasProperty('csv_delimiter') )        
        
    def testModificationsToPropSheetNotOverwritten(self):
        newprop = 'foo'
        self.properties.ploneformgen_properties.manage_changeProperties(mail_body_type=newprop)
        
        # reinstall
        qi = self.portal.portal_quickinstaller
        qi.reinstallProducts(['PloneFormGen'])
        
        # make sure we still have our new value for 'mail_body_type'
        self.assertEquals(newprop, self.properties.ploneformgen_properties.getProperty('mail_body_type'))
    
    def testModificationsToPropSheetLinesNotPuged(self):
        pfg_property_mappings = [
            {"propsheet":"navtree_properties",
             "added_props":["metaTypesNotToList",]},
            {"propsheet":"ploneformgen_properties",
             "added_props":["permissions_used", "mail_cc_recipients", 
                "mail_bcc_recipients", "mail_xinfo_headers","mail_add_headers", "csv_delimiter"]},
            {"propsheet":"site_properties",
             "added_props":["use_folder_tabs", "typesLinkToFolderContentsInFC",
                "types_not_searched", "default_page_types"]},
        ]
        
        # add garbage prop element to each lines property
        for mapping in pfg_property_mappings:
            sheet = self.properties[mapping['propsheet']]
            for lines_prop in mapping['added_props']:
                propitems = list(sheet.getProperty(lines_prop))
                propitems.append('foo')
                sheet.manage_changeProperties({lines_prop:propitems})
        
        # reinstall
        qi = self.portal.portal_quickinstaller
        qi.reinstallProducts(['PloneFormGen'])
        
        # now make sure our garbage values survived the reinstall
        for mapping in pfg_property_mappings:
            sheet = self.properties[mapping['propsheet']]
            for lines_prop in mapping['added_props']:
                self.failUnless('foo' in sheet.getProperty(lines_prop),
                    "Our garbage item didn't survive reinstall for property %s"
                    " within property sheet %s" % (lines_prop, mapping["propsheet"]))
        
    
    def test_FormFolderInDefaultPageTypes(self):
        propsTool = getToolByName(self.portal, 'portal_properties')
        siteProperties = getattr(propsTool, 'site_properties')
        defaultPageTypes = list(siteProperties.getProperty('default_page_types'))
        self.failUnless('FormFolder' in defaultPageTypes)

    # # test no longer necessary now that allowed types are set by interface
    # def test_customAllowedTypesNotPurgedOnReinstall(self):
    #     # add a (bogus) custom type to the list of allowed types
    #     self.types['FormFolder'].allowed_content_types = ['foobar'] + list(self.types['FormFolder'].allowed_content_types)
    #     
    #     # reinstall
    #     qi = self.portal.portal_quickinstaller
    #     qi.reinstallProducts(['PloneFormGen'])
    #     
    #     # make sure our custom type is still in the list
    #     self.failUnless('foobar' in self.types['FormFolder'].allowed_content_types)

        
class TestContentCreation(pfgtc.PloneFormGenTestCase):
    """Ensure content types can be created and edited"""

    fieldTypes = [
        'FormSelectionField',
        'FormMultiSelectionField',
        'FormLabelField',
        'FormDateField',
        'FormLinesField',
        'FormIntegerField',
        'FormBooleanField',
        'FormPasswordField',
        'FormFixedPointField',
        'FormStringField',
        'FormTextField',
        'FormRichTextField',
        'FormFileField',
    ]
    if pfgtc.haveRecaptcha:
        fieldTypes.append('FormCaptchaField')
    fieldTypes = tuple(fieldTypes)

    adapterTypes = (
        'FormSaveDataAdapter',
        'FormMailerAdapter',
    )

    thanksTypes = (
        'FormThanksPage',
    )

    fieldsetTypes = (
        'FieldsetFolder',
    )

    sampleContentIds = ('mailer', 'replyto', 'topic', 'comments', 'thank-you')


    def afterSetUp(self):
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        self.folder.invokeFactory('FormFolder', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')

    def testCreateFormFolder(self):
        self.failUnless('ff1' in self.folder.objectIds())

    def testSampleContent(self):
        # check embedded content
        oi = self.ff1.objectIds()
        for id in self.sampleContentIds:
            self.failUnless(id in oi)

    def testSampleMailerSetup(self):
        self.assertEqual(self.ff1.actionAdapter, ('mailer',))
        self.assertEqual(self.ff1.mailer.replyto_field, 'replyto')
        self.assertEqual(self.ff1.mailer.subject_field, 'topic')
        self.assertEqual(self.ff1.mailer.thanksPage, 'thank-you')

    def testFormFolderCanSetDefaultPage(self):
        self.assertEqual(self.ff1.canSetDefaultPage(), False)

    def testEditFormFolder(self):
        self.ff1.setTitle('A title')
        self.ff1.setDescription('A description')

        self.assertEqual(self.ff1.Title(), 'A title')
        self.assertEqual(self.ff1.Description(), 'A description')

    def testCreateFields(self):
        for f in self.fieldTypes:
            fname = "%s1" % f
            self.ff1.invokeFactory(f, fname)
            self.failUnless(fname in self.ff1.objectIds())

    def testCreateAdapters(self):
        for f in self.adapterTypes:
            fname = "%s1" % f
            self.ff1.invokeFactory(f, fname)
            self.failUnless(fname in self.ff1.objectIds())
            self.failUnless(hasattr(self.ff1[fname], 'onSuccess'))

    def testCreateThanksPages(self):
        for f in self.thanksTypes:
            fname = "%s1" % f
            self.ff1.invokeFactory(f, fname)
            self.failUnless(fname in self.ff1.objectIds())
            self.failUnless(hasattr(self.ff1[fname], 'displayFields'))

    def testCreateFieldset(self):
        for f in self.fieldsetTypes:
            fname = "%s1" % f
            self.ff1.invokeFactory(f, fname)
            self.failUnless(fname in self.ff1.objectIds())

    def testCreateFieldsinFieldset(self):
        fname = 'FieldsetFolder1'
        self.ff1.invokeFactory('FieldsetFolder', fname)
        self.failUnless(fname in self.ff1.objectIds())
        fs = self.ff1[fname]
        for f in self.fieldTypes:
            fname = "%s1fs" % f
            fs.invokeFactory(f, fname)
            self.failUnless(fname in fs.objectIds())

        # Things that shouldn't fit in a fieldset folder
        self.assertRaises(ValueError, fs.invokeFactory, 'FormFolder', 'ffinf')
        self.assertRaises(ValueError, fs.invokeFactory, 'FormThanksPage', 'ffinf')
        self.assertRaises(ValueError, fs.invokeFactory, 'FieldsetFolder', 'ffinf')
        self.assertRaises(ValueError, fs.invokeFactory, 'FormMailerAdapter', 'ffinf')        

        # try finding the fields
        for f in self.fieldTypes:
            fname = "%s1fs" % f
            self.failUnless( self.ff1.findFieldObjectByName(fname) )                


    def testFgFieldsDisplayOnly(self):
        """ Make sure fgFields displayOnly parameter correctly excludes
            labels and fieldset markers
        """

        noExtras = len(self.ff1.fgFields(displayOnly=True))
        wExtras = len(self.ff1.fgFields())
        self.failUnlessEqual(wExtras, noExtras)

        self.ff1.invokeFactory('FieldsetFolder', 'FieldsetFolder1')
        self.ff1['FieldsetFolder1'].invokeFactory('FormStringField', 'fsf')
        
        noExtras = len(self.ff1.fgFields(displayOnly=True))
        wExtras = len(self.ff1.fgFields())
        self.failUnlessEqual(wExtras, noExtras+2)    

        self.ff1.invokeFactory('FormLabelField', 'flf')
        noExtras = len(self.ff1.fgFields(displayOnly=True))
        wExtras = len(self.ff1.fgFields())
        self.failUnlessEqual(wExtras, noExtras+3)
    
        self.ff1.invokeFactory('FormRichLabelField', 'frlf')
        noExtras = len(self.ff1.fgFields(displayOnly=True))
        wExtras = len(self.ff1.fgFields())
        self.failUnlessEqual(wExtras, noExtras+4)
    
    
    def testEditField(self):
        for f in self.fieldTypes:
            fname = "%s1" % f
            self.ff1.invokeFactory(f, fname)
            f1 = getattr(self.ff1, fname)
            f1.setTitle('Field title')
            f1.setDescription('Field description')

            self.assertEqual(f1.Title(), 'Field title')
            self.assertEqual(f1.fgField.widget.label, 'Field title')
            self.assertEqual(f1.Description(), 'Field description')
            self.assertEqual(f1.fgField.widget.description, 'Field description')

    def testTALESFieldValidation(self):
        for f in self.fieldTypes:
            if f != 'FormLabelField':
                fname = "%s1" % f
                self.ff1.invokeFactory(f, fname)
                f1 = getattr(self.ff1, fname)
                self.assertEqual(f1.getFgTValidator(), False)
                f1.setFgTValidator('python:True')
                self.assertEqual(f1.getFgTValidator(), True)

    def testEditAdapter(self):
        for f in self.adapterTypes:
            fname = "%s1" % f
            self.ff1.invokeFactory(f, fname)
            f1 = getattr(self.ff1, fname)
            f1.setTitle('title')
            f1.setDescription('description')

            self.assertEqual(f1.Title(), 'title')
            self.assertEqual(f1.Description(), 'description')

    def testMailerZPTBody(self):
        fname = 'FormMailerAdapter1'
        self.ff1.invokeFactory('FormMailerAdapter', fname)
        f1 = getattr(self.ff1, fname)
        self.failUnless( f1.getBody_pt(fields=[], wrappedFields=[]) )

    def testEditThanksPages(self):
        for f in self.thanksTypes:
            fname = "%s1" % f
            self.ff1.invokeFactory(f, fname)
            f1 = getattr(self.ff1, fname)
            f1.setTitle('title')
            f1.setDescription('description')

            self.assertEqual(f1.Title(), 'title')
            self.assertEqual(f1.Description(), 'description')

    def testCreateFieldsAdaptersOutsideFormFolder(self):
        for f in self.fieldTypes + self.adapterTypes + self.thanksTypes + self.fieldsetTypes:
            self.assertRaises(ValueError, self.folder.invokeFactory, f, 'f1')

    def testBadIdField(self):
        # test for tracker #32 - Field with id 'language' causes problems with PTS
        
        from Products.CMFCore.exceptions import BadRequest

        fname = 'test_field'
        self.ff1.invokeFactory('FormStringField', fname)
        f1 = getattr(self.ff1, fname)

        self.assertRaises(BadRequest, f1.setId, 'language')

        # also not such a good idea ...
        self.assertRaises(BadRequest, f1.setId, 'form')


    def testFieldRename(self):
        """
        renaming a field should change the __name__ attribute
        of the embedded fgField; tracker issue #42
        """
            
        self.ff1.invokeFactory('FormStringField', 'spam_and_eggs')
        self.failUnless('spam_and_eggs' in self.ff1.objectIds())

        myField = getattr(self.ff1, 'spam_and_eggs')
        fgf = getattr(myField, 'fgField')
        self.assertEqual(fgf.__name__, 'spam_and_eggs')
            
        # XXX TODO: figure out what's wrong with this:
        #self.ff1.manage_renameObject('spam_and_eggs', 'spam_spam_and_eggs')
        #self.assertEqual(fgf.__name__, 'spam_spam_and_eggs')


    def testFieldsetRename(self):
        """
        renaming a fieldset should change the __name__ attribute
        of the embedded fsStartField
        """
            
        self.ff1.invokeFactory('FieldsetFolder', 'fsfolder1')
        self.failUnless('fsfolder1' in self.ff1.objectIds())

        myField = getattr(self.ff1, 'fsfolder1')
        fgf = getattr(myField, 'fsStartField')
        self.assertEqual(fgf.__name__, 'fsfolder1')
            

    def testFieldsetPlusDisplayList(self):
        """ Test for issue  #44 -- Presence of fieldset causes an attribute error
        """

        # create fieldset
        self.ff1.invokeFactory('FieldsetFolder', 'fsf1')
        self.failUnless( self.ff1.fgFieldsDisplayList() )
        

    def testUtfInFieldTitle(self):
        """ test for issue # 102, 104: utf8, non-ascii in field title or description
        """

        self.ff1.invokeFactory('FormStringField', 'sf1',
         title='Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8'))

        self.ff1.sf1.setDescription( 'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8') )
        # force a reindex
        self.ff1.sf1.reindexObject()
        

    def testUtfInFormTitle(self):
        """ test for utf8, non-ascii in form title or description
        """

        self.folder.invokeFactory('FormFolder', 'ff2',
         title='Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8'))

        self.folder.ff2.setDescription( 'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8') )
        # force a reindex
        self.folder.ff2.reindexObject()

    def testBadIds(self):
        """ test ids that cause problems.
            We shouldn't be able to create objects with ids known
            to be troublesome.
            Also, all fields in all fieldsets must have unique ids.
        """

        # should be OK:
        self.failUnless( self.ff1.checkIdAvailable('somethingunique8723') )
        
        # bad ids should fail
        self.failIf( self.ff1.checkIdAvailable('zip') )
        self.failIf( self.ff1.checkIdAvailable('location') )
        self.failIf( self.ff1.checkIdAvailable('language') )

        # existing ids should fail
        self.ff1.invokeFactory('FormStringField', 'sf1')
        self.failIf( self.ff1.checkIdAvailable('sf1') )

        # test in fieldset folder
        self.ff1.invokeFactory('FieldsetFolder', 'fsf1')
        fsf1 = self.ff1.fsf1
        self.failUnless( fsf1.checkIdAvailable('somethingunique8723') )
        self.failIf( fsf1.checkIdAvailable('zip') )
        
        # We should also not be able to create a fieldset folder field
        # with an id the same as one in the parent form folder
        self.failIf( fsf1.checkIdAvailable('sf1') )
        # nor in the fieldset folder itself
        fsf1.invokeFactory('FormStringField', 'sf2')
        self.failIf( fsf1.checkIdAvailable('sf2') )
        
        # Let's try it in a sibling fieldset folder
        self.ff1.invokeFactory('FieldsetFolder', 'fsf2')
        fsf2 = self.ff1.fsf2
        self.failUnless( fsf2.checkIdAvailable('somethingunique8723') )
        self.failIf( fsf2.checkIdAvailable('sf1') )
        self.failIf( fsf2.checkIdAvailable('sf2') )
        
        


class TestGPG(pfgtc.PloneFormGenTestCase):
    """ test ya_gpg.py """
    
    def test_gpg(self):
        from Products.PloneFormGen.content.ya_gpg import gpg, GPGError
        
        if gpg is None:
            print "\nSkipping GPG tests; gpg binary not found"
        else:
            self.assertRaises(GPGError, gpg.encrypt, 'spam', 'eggs')

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestContentCreation))
    suite.addTest(makeSuite(TestGPG))
    return suite



__author__ = """Titus Anderson <titus.anderson@louisville.edu>"""
__docformat__ = 'plaintext'

# Copyright (c) 2007 by Copyright (c) 2007 Titus Anderson
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#


from Products.Archetypes.public import *
from Products.Archetypes.utils import shasattr

from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.configuration import zconf

from Products.Archetypes.Widget import TypesWidget

from Products.Archetypes.Registry import registerWidget

from Products.CMFCore.permissions import View, ModifyPortalContent

from AccessControl import ClassSecurityInfo

class LikertWidget(TypesWidget):
    _properties = TypesWidget._properties.copy()
    _properties.update({
        'macro' : 'widget_likert'
    })

    def process_form(self, instance, field, form, empty_marker=None, emptyReturnsMarker=False):

        fieldName = field.getName()
        questions = field.getQuestionSet()

        answers = []

        frec = form.get(fieldName, {})
        for index in range(len(questions)):
            value = frec.get(str(index + 1), empty_marker)
            answers.append(value)

        return answers, {}
        
registerWidget(LikertWidget,
               title='Likert Widget',
               description='Renders a Likert question and answer box',
               used_for=('Products.PloneFormGen.content.likertField.LikertField',),
)




# Import the base test case classes
from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase

from Products.Five import fiveconfigure
from Products.Five import zcml
from Products.PloneTestCase.layer import onsetup
import Products.PloneFormGen
try:
    import collective.recaptcha
    haveRecaptcha = True
except ImportError:
    haveRecaptcha = False
    print "collective.recaptcha is unavailable: captcha tests will be skipped."    

from Products.Five.testbrowser import Browser

ZopeTestCase.installProduct('PloneFormGen')

@onsetup
def setup_product():
    fiveconfigure.debug_mode = True
    zcml.load_config('configure.zcml', Products.PloneFormGen)
    if haveRecaptcha:
        zcml.load_config('configure.zcml', collective.recaptcha)
    fiveconfigure.debug_mode = False

# Set up the Plone site used for the test fixture. The PRODUCTS are the products
# to install in the Plone site (as opposed to the products defined above, which
# are all products available to Zope in the test fixture)
setup_product()
PloneTestCase.setupPloneSite(products=['PloneFormGen'])

class Session(dict):
    def set(self, key, value):
        self[key] = value

from Products.SecureMailHost.SecureMailHost import SecureMailHost
class MailHostMock(SecureMailHost):
    def _send(self, mfrom, mto, messageText):
        print '<sent mail from %s to %s>' % (mfrom, mto)
        self.msgtext = messageText

class PloneFormGenTestCase(PloneTestCase.PloneTestCase):
    def _setup(self):
        # make sure we test in Plone 2.5 with the exception hook monkeypatch applied
        Products.PloneFormGen.config.PLONE_25_PUBLISHER_MONKEYPATCH = True
        
        PloneTestCase.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = Session()

class PloneFormGenFunctionalTestCase(PloneTestCase.FunctionalTestCase):

    def _setup(self):
        PloneTestCase.FunctionalTestCase._setup(self)
        self.app.REQUEST['SESSION'] = Session()
        self.browser = Browser()
        self.app.acl_users.userFolderAddUser('root', 'secret', ['Manager'], [])
        self.browser.addHeader('Authorization', 'Basic root:secret')
        self.portal_url = 'http://nohost/plone'
        
    def afterSetUp(self):
        super(PloneTestCase.FunctionalTestCase, self).afterSetUp()
        self.portal.MailHost = MailHostMock()

    def setStatusCode(self, key, value):
        from ZPublisher import HTTPResponse
        HTTPResponse.status_codes[key.lower()] = value
